package peer2me.framework;

import javax.bluetooth.BluetoothStateException;
import javax.bluetooth.LocalDevice;

import peer2me.exception.FrameworkNotInitializedException;
import peer2me.exception.GroupNotFoundException;
import peer2me.exception.NodeNotFoundException;
import peer2me.group.Group;
import peer2me.message.Message;
import peer2me.network.Network;
import peer2me.node.Node;
import peer2me.service.Service;
import peer2me.session.Session;
import peer2me.util.FileHandler;

/**
 * Applications using the Peer2Me framework will interact mostly with this class. After this
 * class is instanciated, it creates an instance of Session, initializes the Log, sets the Frame-
 * workSubscriber and the FileHandler.
 * This class will provide methods for most operations within the framework, beeing the link
 * between the application and the Peer2Me framework. 
 * @author Tommy Bjoernsgaard
 *
 */
public class Framework 
{
	private static Framework singleton = null;
	
	private FrameworkSubscriber frameworkSubscriber;
	private Session session;
	private FileHandler fileHandler;
	
	/**
	 * This method will create a new or get an existing instance of Framework. Uses singelton.
	 * Will notify session of the groupname, servicename and network. Will also set the
	 * {@link peer2me.framework.FrameworkSubscriber}
	 * @param groupName the name of the default group
	 * @param serviceName the name of the service: serviceID
	 * @param network the network to be used
	 * @param activatePing sets if ping-functionality should be used
	 * @param frameworkSubscriber the frameworksubscriber
	 * @return a reference to Framework
	 */
	public static synchronized Framework getInstance(String groupName, String serviceName, Network network, boolean activatePing, FrameworkSubscriber frameworkSubscriber)
	{
		if(singleton == null)
		{
			singleton = new Framework();
			singleton.setSession(Session.getInstance(singleton.getFramework(), new Group(groupName),new Service(serviceName),network, activatePing));
			singleton.setframeworkSubscriber(frameworkSubscriber);
			singleton.setFileHandler(FileHandler.getInstance());
		}		
		return singleton;
	}
	
	/**
	 * Default private constructor
	 *
	 */
	private Framework()
	{
	}
	
	/**
	 * Gets a local reference to this class which is passed on to session.
	 * @return
	 */
	private Framework getFramework()
	{
		return this;
	}
	
	/**
	 * Sets a reference to {@link peer2me.session.Session}.
	 * @param session the session-reference
	 */
	private void setSession(Session session)
	{
		this.session = session;
	}
	
	/**
	 * Cleans up resources used by the framework. Closes all active connections.
	 *
	 */
	public void clean()
	{
		session.getNetwork().clean();
	}
	
	/**
	 * Gets a reference to {@link peer2me.session.Session}
	 * @return the reference to session
	 */
	public Session getSession()
	{
		return session;
	}
	
	/**
	 * This method initializes the framework, calling
	 * on <code>initialize()</code> in {@link peer2me.session.Session}
	 *
	 */
	public void initialize()
	{
		session.initialize();	
	}
	
	/**
	 * This methods starts a search for devices and services. Initialize must be
	 * called before trying to perform a search. Calls on <code>search()</code> in
	 * {@link peer2me.session.Session}
	 * @throws FrameworkNotInitializedException
	 */
	public void search() throws FrameworkNotInitializedException
	{
		session.search();
	}
	
	/**
	 * Gets all the groups. Calling on the corresponding method in
	 * {@link peer2me.session.Session}
	 * @return an array of the groups
	 */
	public Group[] getGroups()
	{
		return session.getGroups();
	}
	
	/**
	 * Gets the {@link peer2me.group.Group} at the given index in the collection of
	 * groups in session. Throws an exception if the group is not found.
	 * Calling on the corresponding method in
	 * {@link peer2me.session.Session}.
	 * @param index the index of the group
	 * @return the group found at the index
	 * @throws GroupNotFoundException
	 */
	public Group getGroupAt(int index) throws GroupNotFoundException
	{
		return session.getGroupAt(index);
	}

	/**
	 * Gets a {@link peer2me.group.Group} with the corresponding name. Throws an exception if
	 * the group is not found.
	 * Calling on the corresponding method in
	 * {@link peer2me.session.Session}.
	 * @param name the name of the group
	 * @return the group
	 * @throws GroupNotFoundException
	 */
	public Group getGroup(String name) throws GroupNotFoundException
	{
		return session.getGroup(name);
	}

	/**
	 * This method creates a new {@link peer2me.group.Group} and adds it to 
	 * the groupcollection session.
	 * Calling on the corresponding method in
	 * {@link peer2me.session.Session}.
	 * @param name name of the group
	 * @return the group just created
	 */
	public Group createGroup(String name)
	{
		return session.createGroup(name);
	}
	
	/**
	 * This method creates a {@link peer2me.group.Group} and adds it to
	 * the session. Adds also an array of nodes directly.
	 * Calling on the corresponding method in
	 * {@link peer2me.session.Session}.
	 * @param name the name of the group
	 * @param nodes the nodes added upon creation of the group
	 * @return the group just created
	 */
	public Group createGroup(String name, Node[] nodes)
	{
		return session.createGroup(name,nodes);
	}

	/**
	 * Removes the {@link peer2me.group.Group} at the given index. Will also
	 * remove the nodes in the group if not empty. Throws an exception if the group
	 * is not found.
	 * Calling on the corresponding method in
	 * {@link peer2me.session.Session}.
	 * @param index the index of the group
	 * @throws GroupNotFoundException
	 */
	public void removeGroup(int index) throws GroupNotFoundException
	{
		session.removeGroup(index);
	}

	/**
	 * Removes the {@link peer2me.group.Group} with the given name. Will also
	 * remove the nodes in the group if not empty. Throws an exception if the group
	 * is not found.
	 * Calling on the corresponding method in
	 * {@link peer2me.session.Session}.
	 * @param name the name of the group
	 * @throws GroupNotFoundException
	 */
	public void removeGroup(String name) throws GroupNotFoundException
	{
		session.removeGroup(name);
	}

	/**
	 * Gets all the discovered nodes
	 * Calling on the corresponding method in
	 * {@link peer2me.session.Session}.
	 * @return an array of the nodes
	 */
	public Node[] getAllNodes()
	{
		return session.getAllNodes();
	}

	/**
	 * Gets alle the nodes in the given group.
	 * Calling on the corresponding method in
	 * {@link peer2me.session.Session}.
	 * @param group the group of the nodes
	 * @return an array of nodes
	 */
	public Node[] getAllNodes(Group group)
	{
		return session.getAllNodes(group);
	}

	/**
	 * Gets a {@link peer2me.node.Node} based on an address
	 * Calling on the corresponding method in
	 * {@link peer2me.session.Session}.
	 * @param address the address of the node
	 * @return the node with the given address
	 */
	public Node getNode(String address) throws NodeNotFoundException
	{
		return session.getNode(address);
	}
	
	/**
	 * Gets the {@link peer2me.node.Node} at the given <code>index</code>
	 * in the given {@link peer2me.group.Group}. Throws an exception if
	 * the node is not found.
	 * Calling on the corresponding method in
	 * {@link peer2me.session.Session}.
	 * @param index the index of the node
	 * @param group the group of the node
	 * @return the node
	 * @throws NodeNotFoundException
	 */
	public Node getNode(int index, Group group) throws NodeNotFoundException
	{
		return session.getNode(index, group);
	}

	/**
	 * Moves a {@link peer2me.node.Node} from one {@link peer2me.group.Group}
	 * to another. Will also send a notification to all other nodes about the
	 * movement. 
	 * Calling on the corresponding method in
	 * {@link peer2me.session.Session}.
	 * @param node the node to move
	 * @param from the group the node lies in
	 * @param to the new group of the node
	 */
	public void moveNode(Node node, Group from, Group to) throws NodeNotFoundException
	{
		session.moveNode(node,from,to);
	}
	
	/**
	 * Removes a {@link peer2me.node.Node} with the given <code>index</code> from a {@link peer2me.group.Group}. 
	 * Throws an exception if the node is not found.
	 * Calling on the corresponding method in
	 * {@link peer2me.session.Session}.
	 * @param index the index of the node
	 * @param group the group of node
	 * @throws NodeNotFoundException
	 */
	public void removeNode(int index, Group group) throws NodeNotFoundException
	{
		session.removeNode(index, group);
	}
	
	/**
	 * Removes a {@link peer2me.node.Node} from a {@link peer2me.group.Group}.
	 * Calling on the corresponding method in
	 * {@link peer2me.session.Session}.
	 * @param node the node to remove
	 * @param group the group of the node
	 */
	public void removeNode(Node node, Group group)
	{
		session.removeNode(node, group);
	}

	/**
	 * Sets the frameworksubscriber
	 * @param frameworkSubscriber a reference to the frameworksubscriber
	 */
	public void setframeworkSubscriber(FrameworkSubscriber frameworkSubscriber)
	{
		this.frameworkSubscriber = frameworkSubscriber;
	}
	
	/**
	 * This method is called whenever a new {@link peer2me.node.Node} is discovered.
	 * Uses {@link peer2me.framework.FrameworkSubscriber} to communicate with the MIDlet.
	 * @param node the node that is found
	 */
	public void nodeFound(Node node)
	{
		frameworkSubscriber.nodeDiscovered(node);
	}
	
	/**
	 * This method is called whenever a new {@link peer2me.node.Node} is lost or abscent.
	 * Uses {@link peer2me.framework.FrameworkSubscriber} to communicate with the MIDlet.
	 * @param node the node that is lost or abscent
	 */
	public void nodeLost(Node node)
	{
		frameworkSubscriber.nodeLost(node);
	}
	
	/**
	 * This method is called whenever a search for devices and services is finished.
	 * Uses {@link peer2me.framework.FrameworkSubscriber} to communicate with the MIDlet.
	 * 
	 */
	public void searchCompleted()
	{
		frameworkSubscriber.searchCompleted();
	}
	
	/**
	 * This method is called whenever a new {@link peer2me.message.Message} is received.
	 * Uses {@link peer2me.framework.FrameworkSubscriber} to communicate with the MIDlet.
	 * 
	 */
	public void messageReceived(Message message)
	{
		frameworkSubscriber.messageReceived(message);
	}
	
	/**
	 * This method is called whenever new {@link peer2me.message.MessagePart} is received.
	 * This method is useful when creating progressbars etc.
	 * Uses {@link peer2me.framework.FrameworkSubscriber} to communicate with the MIDlet.
	 * @param part the part received of a total
	 * @param total the total number of parts to receive
	 */
	public void messagePartReceived(String messageID, int part, int total)
	{
		frameworkSubscriber.messagePartReceived(messageID,part,total);
	}
	
	/**
	 * Passes on a {@link peer2me.message.Message} to {@link Session}. Used when sending messages.
	 * Calling on the corresponding method in
	 * {@link peer2me.session.Session}.
	 * @param message the message to be sent.
	 */
	public void sendMessage(Message message)
	{
		session.sendMessage(message);
	}

	/**
	 * Gets a reference of {@link peer2me.util.FileHandler}
	 * @return a object of the type filehandler
	 */
	public FileHandler getFileHandler()
	{
		return fileHandler;
	}

	/**
	 * Sets a reference of {@link peer2me.util.FileHandler}
	 * @param fileHandler the reference of filehandler
	 */
	public void setFileHandler(FileHandler fileHandler)
	{
		this.fileHandler = fileHandler;
	}
	
	/**
	 * Sets the path of the default folder in which 
	 * received files are stored. Calls on the corresponding
	 * method in {@link peer2me.util.FileHandler}
	 * @param path the absolute path of the folder
	 */
	public void setReceivedFilesFolder(String path)
	{
		this.fileHandler.setReceivedFilesFolder(path);
	}
	
	/**
	 * A simple method which returns a friendly <code>String</code> with
	 * the name and address of the local node.
	 * @return a String with the name and address
	 */
	public String whoAmI()
	{
		try
		{
			return LocalDevice.getLocalDevice().getFriendlyName()+" at "+LocalDevice.getLocalDevice().getBluetoothAddress()+"\n";
		} 
		catch (BluetoothStateException e)
		{
			return "N/A";
		}
	}
	
	/**
	 * Returns the local node
	 * @return The local node
	 */
	public Node getLocalNode()
	{
		return session.getLocalNode();
	}
}
