package peer2me.group;

import java.util.Vector;

import peer2me.exception.NodeNotFoundException;
import peer2me.node.Node;

/**
 * A simple class for storing information
 * about a group and its nodes.
 * Can contain several instances of {@link peer2me.node.Node}. 
 * @author Tommy Bjoernsgaard
 *
 */
public class Group 
{
	private Vector nodecollection;	
	private String name;
	
	/**
	 * Creates a new instance of Group.
	 * @param name the name of the group
	 */
	public Group(String name)
	{
		nodecollection = new Vector();
		this.name = name;
	}
	
	/**
	 * Creates a new instance of Group with a name and existing nodes
	 * @param name the name of the group
	 * @param nodes an array of nodes to put in the group
	 */
	public Group(String name, Node[] nodes)
	{
		Vector v = new Vector();
		for (int i = 0; i < nodes.length; i++)
			v.addElement(nodes[i]);
		this.nodecollection = v;
		v = null;
		this.name = name;
	}

	/**
	 * Adds a {@link peer2me.node.Node} the group
	 * @param node The node to add
	 */
	public void addNode(Node node)
	{
		nodecollection.addElement(node);
	}
	
	/**
	 * Gets an array of {@link peer2me.node.Node} in the group
	 * @return an array of Node with all the nodes in the group
	 */
	public Node[] getNodes()
	{
		Node[] n = new Node[nodecollection.size()];
		nodecollection.copyInto(n);
		return n;
	}
	
	/**
	 * Gets a {@link peer2me.node.Node} at det specified index
	 * @param index inded of the node
	 * @return the node at the specified index
	 */
	public Node getNodeAt(int index) throws NodeNotFoundException
	{
		if(index < nodecollection.size())
			return (Node)nodecollection.elementAt(index);
		throw new NodeNotFoundException();
	}
	
	/**
	 * Gets a {@link peer2me.node.Node} with the specified address
	 * @param address the address of the node
	 * @return the node with specified address
	 * @throws NodeNotFoundException
	 */
	public Node getNode(String address) throws NodeNotFoundException
	{
		for (int i = 0; i < nodecollection.size(); i++)
		{
			if(((Node)nodecollection.elementAt(i)).getAddress().equals(name))
				return (Node)nodecollection.elementAt(i);
		}
		throw new NodeNotFoundException();
	}
	
	/**
	 * Returns the index of a given {@link peer2me.node.Node}. Useful when finding out if a node exists in this
	 * group
	 * @param node the node to find the index of
	 * @return index of the node give as parameter
	 */
	public int getIndexOfNode(Node node)
	{
		return nodecollection.indexOf(node);
	}
	
	/**
	 * Returns the number of nodes in the group.
	 * @return An integer representing the number of nodes
	 */
	public int getNumberOfNodes()
	{
		return nodecollection.size();
	}
	
	/**
	 * Removes a {@link peer2me.node.Node} at the specified index
	 * @param index index of the node
	 */
	public void removeNodeAt(int index) throws NodeNotFoundException
	{
		if(index < nodecollection.size())
			nodecollection.removeElementAt(index);
		else
			throw new NodeNotFoundException();
	}
	
	/**
	 * Removes a {@link peer2me.node.Node} with the given address
	 * @param address the address of the node
	 * @throws NodeNotFoundException
	 */
	public void removeNode(String address) throws NodeNotFoundException
	{
		boolean foundNode = false;
		for (int i = 0; i < nodecollection.size(); i++)
		{
			if(((Node)nodecollection.elementAt(i)).getAddress().equals(name))
			{
				nodecollection.removeElementAt(i);
				foundNode = true;
			}
		}
		if(!foundNode)
			throw new NodeNotFoundException();
	}
	
	/**
	 * Removes the specified {@link peer2me.node.Node}
	 * @param node the node to remove
	 */
	public void removeNode(Node node)
	{
		nodecollection.removeElement(node);
	}
	
	/**
	 * Removes all nodes from the group
	 */
	public void removeAllNodes()
	{
		nodecollection.removeAllElements();
	}
	
	/**
	 * Gets the name of the group
	 * @return name of the group
	 */
	public String getName()
	{
		return name;
	}
	
	/**
	 * Alters the name of the group
	 * @param name the new name
	 */
	public void setName(String name)
	{
		this.name = name;
	}	
}
