package peer2me.log;

import java.util.Vector;

import javax.microedition.lcdui.List;

/**
 * A class for handling the information and feedback
 * given by the framework. Gives the user/developer
 * a response system used to present information and
 * debug-options.
 * Embeds enumeration-funcionality and exposes methods for
 * handling the log.
 * ShowDate is default set to <code>false</code> 
 * @author Tommy Bjoernsgaard
 *
 */
public class Log
{
	private static Log singleton = null;
	private Vector logEntries = new Vector();
	private boolean showDate = false;
	private boolean showTime = true;	
	private boolean turnedOn = true;	
	private int counter = 0;
	
	/**
	 * Gets the current or creates a new instance of
	 * the log. In the first run, showDate is <code>false</code>
	 * @return an object of Log
	 */
	public static synchronized Log getInstance()
	{
		if(singleton == null)
		{
			singleton = new Log();
			singleton.addElement(LogElementType.INFORMATION,"Created Log");
		}		
		return singleton;
	}
	
	/**
	 * Sets if the date should show in the log-entry
	 * @param showDate true if should show, else false
	 */
	public void showDate(boolean showDate)
	{
		this.showDate = showDate;
	}
	
	/**
	 * Sets if the time should show in the log-entry
	 * @param showTime true if the time should show, else false
	 */
	public void showTime(boolean showTime)
	{
		this.showTime = showTime;
	}
	
	/**
	 * Gets true if date is to be shown, else false
	 * @return The value indicating if date is shown
	 */
	public boolean isShowDate()
	{
		return showDate;
	}
	
	/**
	 * Gets true if time is to be shown, else false
	 * @return The value indicating if time is shown
	 */
	public boolean isShowTime()
	{
		return showTime;
	}
	
	/**
	 * Adds a new entry to the log
	 * @param type The type of entry, a {@link peer2me.log.LogElementType}
	 * @param text A description of the entry
	 */
	public void addElement(LogElementType type, String text)
	{
		if(turnedOn)
		{
			logEntries.addElement(new LogElement(type,text,showDate,showTime));
		}
	}
	
	/**
	 * Gets a log-entry at the specified index.
	 * @param index The index of the log-entry
	 * @return The LogElement at the given index
	 */
	public LogElement getElementAt(int index)
	{
		return (LogElement)logEntries.elementAt(index);
	}

	/**
	 * Gets the first element added to the log. 
	 * @return The first {@link peer2me.log.LogElement} in the log
	 */
	public LogElement getFirstElement()
	{
		return (LogElement)logEntries.firstElement();
	}
	
	/**
	 * Gets the last element added to the log.
	 * @return The last {@link peer2me.log.LogElement} in the log
	 */
	public LogElement getLastElement()
	{
		return (LogElement)logEntries.lastElement();
	}
	
	/**
	 * Gets the next {@link peer2me.log.LogElement} from the log.
	 * The pointer is then shifted on to the next 
	 * element. If there are no more elements, <code>null</code> is returned.
	 * @return The next LogElement in line, or <code>null</code>
	 */
	public LogElement getNextElement()
	{
		counter++;
		if(logEntries.size() == counter)
			return null;
		return (LogElement)logEntries.elementAt(counter-1);	
	}
	
	/**
	 * Gets the previous {@link peer2me.log.LogElement} from the log.
	 * The pointer is then shifted on to the 
	 * element before the element returned. If there are no more elements, <code>null</code> is returned.
	 * @return The previous LogElement, or <code>null</code>
	 */
	public LogElement getPreviousElement()
	{
		counter--;
		if(counter == -1)
			return null;
		return (LogElement)logEntries.elementAt(counter);	
	}
	
	/**
	 * Gets all the log-entries as an array
	 * of {@link peer2me.log.LogElement}
	 * @return The array of LogElement
	 */
	public LogElement[] getElements()
	{
		LogElement[] elements = new LogElement[logEntries.size()];
		logEntries.copyInto(elements);
		return elements;
	}
	/**
	 * Gets all the log-entries as an array
	 * of {@link peer2me.log.LogElement} based on the {@link peer2me.log.LogElementType}
	 * @param type the type of log-entry to get
	 * @return The array of LogElement
	 */
	public LogElement[] getElements(LogElementType type)
	{
		Vector v = new Vector();
		for(int i=0;i<logEntries.size();i++)
			if(((LogElement)logEntries.elementAt(i)).getType().equals(type))
				v.addElement((LogElement)logEntries.elementAt(i));

		LogElement[] elements = new LogElement[v.size()];
		v.copyInto(elements);
		v = null;
		return elements;
	}
	
	/**
	 * Creates a<a href="http://www.j2medev.com/api/midp/javax/microedition/lcdui/List.html">javax.microedition.lcdui.List</a> based on
	 * the elements in the log. Can easily be added to
	 * a <a href="http://www.j2medev.com/api/midp/javax/microedition/lcdui/Display.html">javax.microedition.lcdui.Display</a>
	 * The List is List.IMPLICIT.
	 * @return the List with the elements
	 */
	public List getElementsAsList()
	{
		return createList("Peer2Me Log",null);
	}
	
	/**
	 * Creates a <a href="http://www.j2medev.com/api/midp/javax/microedition/lcdui/List.html">javax.microedition.lcdui.List</a> based on
	 * the the given elementtype and elements in the log. 
	 * a <a href="http://www.j2medev.com/api/midp/javax/microedition/lcdui/Display.html">javax.microedition.lcdui.Display</a>.
	 * The List is List.IMPLICIT.
	 * @param type The type of log-entries to show
	 * @return the List with the elements
	 */
	public List getElementsAsList(LogElementType type)
	{
		return createList("Peer2Me Log",type);
	}
	
	/**
	 * Creates a <a href="http://www.j2medev.com/api/midp/javax/microedition/lcdui/List.html">javax.microedition.lcdui.List</a> based on
	 * the elements in the log. Can easily be added to
	 * a <a href="http://www.j2medev.com/api/midp/javax/microedition/lcdui/Display.html">javax.microedition.lcdui.Display</a>. Custom title.
	 * The List is List.IMPLICIT.
	 * @param title the title of the List
	 * @return the List with the elements
	 */
	public List getElementsAsList(String title)
	{
		return createList(title,null);
	}
	
	/**
	 * Creates a <a href="http://www.j2medev.com/api/midp/javax/microedition/lcdui/List.html">javax.microedition.lcdui.List</a> based on
	 * the the given elementtype and elements in the log. 
	 * a <a href="http://www.j2medev.com/api/midp/javax/microedition/lcdui/Display.html">javax.microedition.lcdui.Display</a>. Custom title
	 * The List is List.IMPLICIT.
	 * @param title the title of the List
	 * @param type The type of log-entries to show
	 * @return the List with the elements
	 */
	public List getElementsAsList(String title, LogElementType type)
	{
		return createList(title,type);
	}
	
	/**
	 * Used to convert log-entries to a List.
	 * Used by the getElementsAsList-methods.
	 * @param title title of the List
	 * @param type type of messages to show
	 * @return the javax.microedition.lcdui.List with the elements 
	 */
	private List createList(String title, LogElementType type)
	{
		LogElement[] entries;
		if(type == null)
			entries = getElements();
		else
			entries = getElements(type);
		
		String[] strings = new String[entries.length];
		for(int i=0;i<strings.length;i++)
			strings[i] = entries[i].toString();
		
		List list = new List(title,List.IMPLICIT,strings,null);
		
		strings = null;
		entries = null;		
		
		return list;
	}

	/**
	 * Gets if the log-functionality is turned on or not
	 * @return ture if the log i active, else false
	 */
	public boolean isTurnedOn()
	{
		return turnedOn;
	}

	/**
	 * Sets if the log should be active
	 * @param turnedOn true to activate the log, else false
	 */
	public void setTurnedOn(boolean turnedOn)
	{
		this.turnedOn = turnedOn;
	}
}
