package peer2me.message;

import java.io.IOException;

import javax.microedition.io.Connector;
import javax.microedition.io.file.FileConnection;

import peer2me.exception.FileNotFoundException;
import peer2me.log.Log;
import peer2me.log.LogElementType;

/**
 * Parent class of the two childclasses SendFileObject and ReceivedFileObject
 * A FileObject can be attached to a Message
 * @author Kim Saxlund
 * @see SendFileObject
 * @see ReceivedFileObject
 * @see Message
 *
 */
public abstract class FileObject
{
	protected String fileName;
	protected String fullPathToFile;
	protected FileConnection fileConnection;
	protected boolean accessGranted;
	
	protected long fileSize;
	
	/**
	 * A constructor that takes the full path of a file including the file itself as an argument
	 * @param fullPathToFile Must be the full path to a file, for instance: 
	 * <code>file:///SDCard/images/animal.jpg</code>
	 * This method will block when {@link javax.microedition.io.Connector#open(java.lang.String)} is called.
	 * The Midlet will then automatically ask the user for permission to access the local filesystem. When 
	 * access is granted, a connection is established.
	 * @throws FileNotFoundException If the file is not found on the local device
	 */
	public FileObject(String fullPathToFile) throws FileNotFoundException
	{
		this.fullPathToFile = fullPathToFile;
		accessGranted = false;
		try
		{
			//Connector.open will block until the user has granted the midlet access to the filesystem
			fileConnection = (FileConnection) Connector.open(this.fullPathToFile);
		} 
		catch (IOException e)
		{
			Log.getInstance().addElement(LogElementType.ERROR,"File not found: " + e.getMessage());
			throw new FileNotFoundException();
		}
	}

	/**
	 * Returns the name of the file that is assosiated with this FileObject
	 * @return The name of the file, for instance: <code>animal.jpg</code>
	 */
	public String getFileName()
	{
		return fileName;
	}

	/**
	 * Returns the size of the file that is assosiated with this FileObject
	 * @return The size of the file in bytes
	 */
	public long getFileSize()
	{
		return fileSize;
	}

	/**
	 * Returns the full path to the file, including the filename
	 * @return The path of the file, for instance: <code>file:///SDCard/images/animal.jpg</code>
	 */
	public String getFullPathToFile()
	{
		return fullPathToFile;
	}
	
	/**
	 * Returns true if the user has granted the Midlet access to the filesystem. False otherwise
	 * @return True if the user has granted the Midlet access to the filesystem. False otherwise
	 */
	public boolean isAccessGranted()
	{
		return accessGranted;
	}
}
