package peer2me.message;

/**
 * This class is a dynamically linked list that is used by the framework to store 
 * messages that are ready to be sent. The list is processed by the 
 * {@link peer2me.network.bluetooth.MessageQueueProcessor} class.
 * 
 * @author Kim Saxlund
 * @see peer2me.network.bluetooth.MessageQueueProcessor
 * @see peer2me.message.Message
 *
 */
public class LinkedMessageList
{
	private MessageListObject head;
	private int size;
	
	/**
	 * Default constructor. Creates an empty list
	 *
	 */
	public LinkedMessageList()
	{
		head = null;
		size = 0;
	}
	
	/**
	 * Checks to see if the list is empty
	 * @return True if empty, false otherwise
	 */
	public boolean isEmpty()
	{
		if(head == null)
			return true;
		return false;
	}
	
	/**
	 * Inserts a {@link MessageListObject} at the beginning of the list
	 * @param message The message to be wrapped in the MessageListObject
	 */
	public void insert(Message message)
	{
		head = new MessageListObject(message, head);
		size++;
	}

	/**
	 * Inserts a {@link MessageListObject} at the end of the list
	 * @param message The message to be wrapped in the MessageListObject
	 */
	public void insertLast(Message message)
	{
        if(isEmpty())
            insert(message);
        else
        {
            MessageListObject tmp = head;
            while(tmp.getNext() != null)
                tmp = tmp.getNext();
            MessageListObject tmp2 = new MessageListObject(message,tmp.getNext());
            tmp.setNext(tmp2);
            size++;
        }
    }
	
	/**
	 * Returns the {@link Message} which is contained in the first element in the list. It the removes
	 * the element from the list and sets the next element as the first element
	 * @return The first message in the list
	 */
	public Message getFirstMessage()
	{
		if(isEmpty())
			return null;
		MessageListObject tmp = head;
		head = head.getNext();
		size--;
		return tmp.getMessage();
	}
	
	/**
	 * Returns the number of messages in the list
	 * @return The number of messages in the list
	 */
	public int getNumberOfMessagesInList()
	{
		return size;
	}
}

/**
 * A private class that is used as elements in the {@link LinkedMessageList}. This class holds a 
 * {@link peer2me.message.Message} object and a reference to another MessageListObject.
 * @author saxlund
 *
 */
class MessageListObject
{
	private Message message;
	private MessageListObject next;
	
	/**
	 * Constructor that creates an empty instance of this class
	 *
	 */
	public MessageListObject()
	{
		message = null;
		next = null;
    }

	/**
	 * Constructor that creates a MessageList object that contains a Message and a reference to 
	 * another instance of MessageListObject. This is the standard way of creating dynamical lists.
	 * @param message The Message that is contained within this class.
	 * @param nextMessage The next MessageListObject
	 */
	public MessageListObject(Message message, MessageListObject nextMessage)
	{
		this.message = message;
		this.next = nextMessage;
	}
   
	/**
	 * Sets the local reference to the next MessageListObject
	 * @param next The object that the reference is made to.
	 */
    public void setNext(MessageListObject next)
    {
        this.next = next;
    }
    
    /**
     * Sets the Message in this class
     * @param message The Message to set
     */
    public void setMessage(Message message)
    {
        this.message = message;
    }
    
    /**
     * Returns the reference to the next MessageListObject 
     * @return The next MessageListObject
     */
    public MessageListObject getNext()
    {
        return next;
    }
    
    /**
     * Returns the Message in this class
     * @return The Message
     */
    public Message getMessage()
    {
        return message;
    }
}
