package peer2me.message;

import peer2me.node.Node;

/**
 * This class is a dynamically linked list that is used by the framework to store 
 * pings that are ready to be sent. The list is processed by the 
 * {@link peer2me.network.bluetooth.PingQueueProcessor} class. The list contains all the nodes 
 * that should be pinged. 
 * 
 * @author Kim Saxlund
 * @see peer2me.node.Node
 * @see peer2me.network.bluetooth.PingQueueProcessor
 */
public class LinkedPingRecipientsList
{	
	private PingRecipientObject head;
	private int size;
	
	/**
	 * Default constructor. Creates an empty list
	 *
	 */
	public LinkedPingRecipientsList()
	{
		head = null;
		size = 0;
	}
	
	/**
	 * Checks to see if the list is empty
	 * @return True if empty, false otherwise
	 */
	public boolean isEmpty()
	{
		if(head == null)
			return true;
		return false;
	}
	
	/**
	 * Inserts a {@link Node} at the beginning of the list
	 * @param node The node to be wrapped in the MessageListObject
	 */
	public void insert(Node node)
	{
		head = new PingRecipientObject(node, head);
		size++;
	}

	/**
	 * Inserts a {@link PingRecipientObject} at the end of the list
	 * @param node The node to be wrapped in the PingRecipientObject
	 */
	public void insertLast(Node node)
	{
        if(isEmpty())
            insert(node);
        else
        {
            PingRecipientObject tmp = head;
            while(tmp.getNext() != null)
                tmp = tmp.getNext();
            PingRecipientObject tmp2 = new PingRecipientObject(node, tmp.getNext());
            tmp.setNext(tmp2);
            size++;
        }
    }
	
	/**
	 * Returns the {@link Node} which is contained in the first element in the list. It the removes
	 * the element from the list and sets the next element as the first element
	 * @return The first node in the list
	 */
	public Node getFirstNode()
	{
		if(isEmpty())
			return null;
		PingRecipientObject tmp = head;
		head = head.getNext();
		size--;
		return tmp.getnode();
	}
	
	/**
	 * Returns the number of nodes in the list
	 * @return The number of nodes in the list
	 */
	public int getNumberOfNodesInList()
	{
		return size;
	}
}
/**
 * A private class that is used as elements in the {@link LinkedPingRecipientsList}. This class holds a 
 * {@link peer2me.node.Node} object and a reference to another PingRecipientObject.
 * @author saxlund
 *
 */
class PingRecipientObject
{
	private Node node;
	private PingRecipientObject next;
	
	/**
	 * Constructor that creates an empty instance of this class
	 *
	 */
	public PingRecipientObject()
	{
		node = null;
		next = null;
    }

	/**
	 * Constructor that creates a PingRecipientObject object that contains a Node and a reference to 
	 * another instance of PingRecipientObject. This is the standard way of creating dynamical lists.
	 * @param node The Node that is contained within this class.
	 * @param nextRecipient The next PingRecipientObject
	 */
	public PingRecipientObject(Node node, PingRecipientObject nextRecipient)
	{
		this.node = node;
		this.next = nextRecipient;
	}
   
	/**
	 * Sets the local reference to the next PingRecipientObject
	 * @param next The object that the reference is made to.
	 */
    public void setNext(PingRecipientObject next)
    {
        this.next = next;
    }
    
    /**
     * Sets the Node in this class
     * @param node The Node to set
     */
    public void setnode(Node node)
    {
        this.node = node;
    }
    
    /**
     * Returns the reference to the next PingRecipientObject 
     * @return The next PingRecipientObject
     */
    public PingRecipientObject getNext()
    {
        return next;
    }
    
    /**
     * Returns the Node in this class
     * @return The Node
     */
    public Node getnode()
    {
        return node;
    }
}
