package peer2me.message;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import peer2me.exception.HexConversionException;
import peer2me.exception.InvalidKeyException;
import peer2me.exception.UnknownKeyException;
import peer2me.group.Group;
import peer2me.node.Node;
import peer2me.service.HexBuilder;
import peer2me.session.Session;
import peer2me.log.Log;
import peer2me.log.LogElementType;

/**
 * This class is used by the framework to exchange messages between nodes in the network. A message
 * can contain many messageparts, which can be serialized objects, primitive datatypes and files. 
 * When adding serialized objects to a message, the object must implement the interface peer2me.util.Serializable.
 * 
 * If a message only contains one serialized object or only primitive datatypes, it can be sent as one 
 * bytestream over the network. However, if the message contains several serialized objects and/or large files,
 * the message must be sent in several "chunks" over the network. In order for the receiving node to 
 * put all these parts together, all chuncks that belongs to the same message must have the same ID, which 
 * in this class is the String <code>messageID</code>. Different messages however, must have different unique
 * IDs.
 * 
 * All constants in this class must be in the range 300-399
 * @see peer2me.message.Serializable
 * @author Kim Saxlund
 *
 */
public class Message
{
	private int messageType;
	/**
	 * The unique identificator of this message
	 */
	private String messageID;
	private Vector recipients;
	private Hashtable messageParts;
	private Hashtable primitiveMessageParts;
	private Node sender;
	

	public static final int MESSAGE_DISCONNECT = 300;
	/**
	 * Identifies a message that contains a file or a part of a file
	 */
	public static final int MESSAGE_FILE = 301;
	/** When an application performs a search for new devices, it will automatically after the search has finished,
	 * send a message that contains information about all nodes to all the nodes the application has found. 
	 * This type of message is identified by this constant.
	 */
	public static final int MESSAGE_NODE_JOIN=302;
	/**
	 *  Identifies a message that contains information about a node that is leaving the network
	 */
	public static final int MESSAGE_NODE_LEFT=303;
	/**
	 *  Identifies a message that contains information that a node has been moved from one group to another
	 */
	public static final int MESSAGE_NODE_MOVE=304;
	/**
	 * Identifies a message that contains primitive types or serialized object
	 */
	public static final int MESSAGE_PRIMITIVES_OR_SERIALIZED_OBJECT = 305;
	
	/**
	 * Constructor that should be used by the application that creates a new message that is to be sent over 
	 * the network. The message will generate a random messageID by using <code>new Date().getTime()</code> 
	 * function combined with the method <code>getHexString</code> in class <code>peer2me.util.HexBuilder</code>.
	 *
	 */
	public Message()
	{
		try
		{
			this.messageID = HexBuilder.getHexString(""+new Date().getTime());
		} 
		catch (HexConversionException e)
		{
			Log.getInstance().addElement(LogElementType.WARNING,"Could not calculate unique message ID.Using default");
			this.messageID = ""+new Date().getTime();
		}
		messageParts = new Hashtable();
		primitiveMessageParts = new Hashtable();
		recipients = new Vector();
		sender = Session.getInstance().getLocalNode();
		messageType = MESSAGE_PRIMITIVES_OR_SERIALIZED_OBJECT;
	}
	
	/**
	 * When a node receives a message, the framework needs to create a new message object since the message
	 * is really sent as pure bytes over the network. The messageID is already created by the sender of the
	 * message and needs to be used by the receiver to identify the message.
	 * @param messageID The unique identificator of this message
	 */
	public Message(String messageID)
	{
		this.messageID = messageID;
		messageParts = new Hashtable();
		recipients = new Vector();
		sender = Session.getInstance().getLocalNode();
		messageType = MESSAGE_PRIMITIVES_OR_SERIALIZED_OBJECT;
	}
	
	/**
	 * Calculates the number of chunks that this message must be split into in order to send it over 
	 * the network
	 * @return The number of chunks needed
	 */
	public int numberOfChunksNeeded()
	{
		int counter = 0;
		MessagePart parts[] = getAllMessageParts();
		for (int i = 0; i < parts.length; i++)
		{
			switch (parts[i].getMessagePartType())
			{
			case MessagePart.SERIALIZEDOBJECT:
				counter++;
				break;
			case MessagePart.ALLPRIMITIVES:
				counter++;
				break;
			case MessagePart.FILEOBJECT:
				counter = counter + ((SendFileObject)parts[i].getFileObject()).numberOfChunksNeeded();
				break;
			default:
				break;
			}
		}
		return counter;
	}
	
	/**
	 * Adds a node to the collection of nodes that are to receive the message.
	 * 
	 * @param node The node that should receive the message
	 * @return Returns true if node is added to collection. Returns false if it already existst in the 
	 * collection
	 */
	public boolean addRecipient(Node node)
	{
		if(node.getAddress() == Session.getInstance().getLocalNode().getAddress())
			return false;
		if(recipients.contains(node))
			return false;
		recipients.addElement(node);
		return true;
	}
	
	/**
	 * Sets an entire group of nodes as recipients of the message
	 * @param group The group of recipients that will receive the message
	 */
	public void addRecipients(Group group)
	{
		addRecipients(group.getNodes());
	}
	
	/**
	 * Sets an array of nodes as recipients of this message
	 * @param nodes An array of nodes that should receive this message
	 */
	public void addRecipients(Node nodes[])
	{
		for(int i = 0; i < nodes.length; i++)
			addRecipient(nodes[i]);
	}
	
	/**
	 * Adds an <code>int</code> to this message
	 * @param key The unique key that the <code>int</code> will be assosiated with
	 * @param value The <code>int</code> value that will be attached to this message
	 */
	public void addElement(String key, int value)
	{
		MessagePart messagePart = new MessagePart(key, value);
		primitiveMessageParts.put(key, messagePart);
	}
	
	/**
	 * Adds a <code>double</code> this message 
	 * @param key The unique key that the <code>double</code> will be assosiated with
	 * @param value The <code>double</code> value that will be attached to this message
	 */
	public void addElement(String key, double value)
	{
		MessagePart messagePart = new MessagePart(key, value);
		primitiveMessageParts.put(key, messagePart);
	}
	
	/**
	 * Adds a <code>char</code> this message 
	 * @param key The unique key that the <code>char</code> will be assosiated with
	 * @param value The <code>char</code> value that will be attached to this message
	 */
	public void addElement(String key, char value)
	{
		MessagePart messagePart = new MessagePart(key, value);
		primitiveMessageParts.put(key, messagePart);
	}
	
	/**
	 * Adds a <code>boolean</code> this message 
	 * @param key The unique key that the <code>boolean</code> will be assosiated with
	 * @param value The <code>boolean</code> value that will be attached to this message
	 */
	public void addElement(String key, boolean value)
	{
		MessagePart messagePart = new MessagePart(key, value);
		primitiveMessageParts.put(key, messagePart);
	}
	
	/**
	 * Adds a <code>String</code> this message 
	 * @param key The unique key that the <code>String</code> will be assosiated with
	 * @param value The <code>String</code> value that will be attached to this message
	 */
	public void addElement(String key, String value)
	{
		MessagePart messagePart = new MessagePart(key, value);
		primitiveMessageParts.put(key, messagePart);
	}
	
	/**
	 * Adds a <code>long</code> this message 
	 * @param key The unique key that the <code>long</code> will be assosiated with
	 * @param value The <code>long</code> value that will be attached to this message
	 */
	public void addElement(String key, long value)
	{
		MessagePart messagePart = new MessagePart(key, value);
		primitiveMessageParts.put(key, messagePart);
	}
	
	/**
	 * Adds a <code>float</code> this message 
	 * @param key The unique key that the <code>float</code> will be assosiated with
	 * @param value The <code>float</code> value that will be attached to this message
	 */
	public void addElement(String key, float value)
	{
		MessagePart messagePart = new MessagePart(key, value);
		primitiveMessageParts.put(key, messagePart);
	}
	
	/**
	 * Adds a <code>short</code> this message 
	 * @param key The unique key that the <code>short</code> will be assosiated with
	 * @param value The <code>short</code> value that will be attached to this message
	 */
	public void addElement(String key, short value)
	{
		MessagePart messagePart = new MessagePart(key, value);
		primitiveMessageParts.put(key, messagePart);
	}
	
	/**
	 * Extracts all the primitive variables from one MessagePart to several MessageParts. This method is invoked
	 * by the <code>BluetoothListener</code> class when a message is received. 
	 *
	 */
	public void extractPrimitivesToSeparateParts()
	{
		MessagePart part = (MessagePart)messageParts.get("primitives");
		if(part != null)
		{
			ByteArrayInputStream bis = new ByteArrayInputStream(part.getObjectBytes());
			DataInputStream dis = new DataInputStream(bis);
			
			try
			{
				int dataType = dis.readInt();
				while(dataType != -1)
				{
					MessagePart newPart = null;
					switch (dataType)
					{
					case MessagePart.INT:
						newPart = new MessagePart(dis.readUTF(),dis.readInt());
						break;
					case MessagePart.DOUBLE:
						newPart = new MessagePart(dis.readUTF(),dis.readDouble());
						break;
					case MessagePart.FLOAT:
						newPart = new MessagePart(dis.readUTF(),dis.readFloat());
						break;
					case MessagePart.SHORT:
						newPart = new MessagePart(dis.readUTF(),dis.readShort());
						break;
					case MessagePart.CHAR:
						newPart = new MessagePart(dis.readUTF(),dis.readChar());
						break;
					case MessagePart.STRING:
						newPart = new MessagePart(dis.readUTF(),dis.readUTF());
						break;
					case MessagePart.BOOLEAN:
						newPart = new MessagePart(dis.readUTF(),dis.readBoolean());
						break;
					case MessagePart.LONG:
						newPart = new MessagePart(dis.readUTF(),dis.readLong());
						break;
					default:
						break;					
					}
					messageParts.put(newPart.getKey(),newPart);
					dataType = dis.readInt();
				}
			}
			catch (IOException e)
			{
				try
				{
					dis.close();
					bis.close();
				} 
				catch (IOException e1)
				{
					Log.getInstance().addElement(LogElementType.WARNING,"Could not close streams in Message");
				}
			}
		}
	}
	
	/**
	 * When adding primitive variables to this message, all the variables are stored as separate 
	 * MessageParts. These parts are stored in a HashMap in this class. When this method is called,
	 * all the stored primitive variables are converted into one MessagePart which is assigned the key: <code>primitives</code>. 
	 * This enables the framework to send all the primitive variables in one chunk. When the message is 
	 * received on the receiving node, the parts are extracted back to the original structure by invoking the 
	 * method <code>extractPrimitvesToSeparateParts</code>
	 * 
	 * This method should only be used by classes inside the peer2me package.
	 *
	 */
	public void storeAllPrimitivesInOnePart()
	{		
		if(primitiveMessageParts.isEmpty())
			return;
		
		Enumeration e = primitiveMessageParts.keys();
		
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DataOutputStream dos = new DataOutputStream(baos);
		byte[] allPrimitives = null;
		try
		{
			while(e.hasMoreElements())
			{
				String key = (String)e.nextElement();
				MessagePart part = (MessagePart)primitiveMessageParts.get(key);
				int tmpType = part.getMessagePartType();
				dos.writeInt(tmpType);
				dos.writeUTF(part.getKey());
				
				switch (tmpType)
				{
				case MessagePart.INT:
					dos.writeInt(part.getIntValue());
					break;
				case MessagePart.DOUBLE:
					dos.writeDouble(part.getDoubleValue());
					break;
				case MessagePart.FLOAT:
					dos.writeFloat(part.getFloatValue());
					break;
				case MessagePart.SHORT:
					dos.writeShort(part.getShortValue());
					break;
				case MessagePart.CHAR:
					dos.writeChar(part.getCharValue());
					break;
				case MessagePart.STRING:
					dos.writeUTF(part.getStringValue());
					break;
				case MessagePart.BOOLEAN:
					dos.writeBoolean(part.getBooleanValue());
					break;
				case MessagePart.LONG:
					dos.writeLong(part.getLongValue());
					break;
				default:
					break;
				}	
			}
			allPrimitives = baos.toByteArray();
			dos.close();
			baos.close();
		} 
		catch (IOException e1)
		{
			Log.getInstance().addElement(LogElementType.ERROR,"Could not store primitives in Message");
		}
		if(allPrimitives != null)
		{
			MessagePart part = new MessagePart("primitives",allPrimitives);
			part.setMessagePartType(MessagePart.ALLPRIMITIVES);
			messageParts.put("primitives",part);
		}
		else
			Log.getInstance().addElement(LogElementType.ERROR,"Error storing all primitives in one bytearray");
	}
	
	/**
	 * Adds a file to this message. This method only adds the path to the file and not the filecontents. 
	 * The content of the file is not read before the message is actually sent. When the framework is
	 * instructed to send the message, the file is read and sent in chunks of sizes specified in the 
	 * <code>SendFileObject</code> class. 
	 * 
	 * @param key The unique key that the file will be assosiated with
	 * @param fullPath The full path to the directory of which the file is located. An example of such a path
	 * is: <code>file:///SDCard/images/animal.jpg</code>
	 */
	public void addFile(String key, String fullPath)
	{
		SendFileObject fileObject = new SendFileObject(fullPath);
		MessagePart messagePart = new MessagePart(key,fileObject);
		messageParts.put(key,messagePart);
	}
	
	/**
	 * When a message containing a file or a part of a file is received, a messapart containing the 
	 * file must be created. This method adds a such a messagepart
	 * 
	 * 
	 * This method should only be used by classes inside the peer2me package.
	 * @param key The unique key that the file will be assosiated with
	 * @param fileObject The fileObject that will be attached to this message
	 */
	public void addReceivedFile(String key, FileObject fileObject)
	{
		MessagePart messagePart = new MessagePart(key,fileObject);
		messageParts.put(key,messagePart);
	}
	
	/**
	 * Should only be used by the framework itself
	 * @param key A unique key that is assosiated with a messagePart
	 * @return The messagPart that is assosiated with the key
	 */
	public MessagePart getMessagePart(String key)
	{
		return (MessagePart)messageParts.get(key);
	}
	
	/**
	 * Returns the <code>int</code> value that is assosiated with the <code>key</code>
	 * @param key A unique key that is assosiated with an <code>int</code> value
	 * @return The <code>int</code> that is assosiated with the <code>key</code>
	 */
	public int getInt(String key)
	{
		return ((MessagePart)messageParts.get(key)).getIntValue();
	}
	
	/**
	 * Returns the <code>double</code> value that is assosiated with the <code>key</code>
	 * @param key A unique key that is assosiated with an <code>double</code> value
	 * @return The <code>double</code> that is assosiated with the <code>key</code>
	 */
	public double getDouble(String key)
	{
		return ((MessagePart)messageParts.get(key)).getDoubleValue();
	}
	
	/**
	 * Returns the <code>String</code> value that is assosiated with the <code>key</code>
	 * @param key A unique key that is assosiated with an <code>String</code> value
	 * @return The <code>String</code> that is assosiated with the <code>key</code>
	 */
	public String getString(String key)
	{
		return ((MessagePart)messageParts.get(key)).getStringValue();
	}
	
	/**
	 * Returns the <code>boolean</code> value that is assosiated with the <code>key</code>
	 * @param key A unique key that is assosiated with an <code>boolean</code> value
	 * @return The <code>boolean</code> that is assosiated with the <code>key</code>
	 */
	public boolean getBoolean(String key)
	{
		return ((MessagePart)messageParts.get(key)).getBooleanValue();
	}
	
	/**
	 * Returns the <code>long</code> value that is assosiated with the <code>key</code>
	 * @param key A unique key that is assosiated with an <code>long</code> value
	 * @return The <code>long</code> that is assosiated with the <code>key</code>
	 */
	public long getLong(String key)
	{
		return ((MessagePart)messageParts.get(key)).getLongValue();
	}
	
	/**
	 * Returns the <code>float</code> value that is assosiated with the <code>key</code>
	 * @param key A unique key that is assosiated with an <code>float</code> value
	 * @return The <code>float</code> that is assosiated with the <code>key</code>
	 */
	public float getFloat(String key)
	{
		return ((MessagePart)messageParts.get(key)).getFloatValue();
	}
	
	/**
	 * Returns the <code>short</code> value that is assosiated with the <code>key</code>
	 * @param key A unique key that is assosiated with an <code>short</code> value
	 * @return The <code>short</code> that is assosiated with the <code>key</code>
	 */
	public short getShort(String key)
	{
		return ((MessagePart)messageParts.get(key)).getShortValue();
	}
	
	/**
	 * Returns the <code>char</code> value that is assosiated with the <code>key</code>
	 * @param key A unique key that is assosiated with an <code>char</code> value
	 * @return The <code>char</code> that is assosiated with the <code>key</code>
	 */
	public char getChar(String key)
	{
		return ((MessagePart)messageParts.get(key)).getCharValue();
	}
	
	/**
	 * Adds information about a received file.
	 * Must not be used to add a file to this Message. If you want to add a file to this Message, 
	 * then you must use the method {@link #addFile(String, String)}
	 * 
	 * This method should only be used by the framework itself
	 * @param key A unique key that is assosiated with the file
	 * @param fileInfo The FileInfo object
	 */
	public void addFileInfo(String key, FileInfo fileInfo)
	{
		MessagePart messagePart = new MessagePart(key,fileInfo);
		messageParts.put(key,messagePart);
	}
	
	/**
	 * Returns a FileInfo object that contains information about the received file.
	 * @param key A unique key that is assosiated with the file
	 * @return The FileInfo object
	 */
	public FileInfo getFile(String key)
	{
		return ((MessagePart)messageParts.get(key)).getFileInfo();
	}
	
	/**
	 * Returns all the MessageParts that are attached to this message
	 * 
	 * Should only be used by the framework itself
	 * @return An array of all the MessageParts that are attached to this message
	 */
	public MessagePart[] getAllMessageParts()
	{
		MessagePart[] parts = new MessagePart[messageParts.size()];
		Enumeration e = messageParts.keys();
		int i = 0;
		while(e.hasMoreElements())
		{
			String key = (String)e.nextElement();
			parts[i] = (MessagePart)messageParts.get(key);
			i++;
		}
		return parts;
	}
	
	/**
	 * Returns all the recipients of this message
	 * @return An array of Nodes that are the recipients of this message
	 */
	public Node[] getRecipients() 
	{
		Node[] nodes = new Node[recipients.size()];
		recipients.copyInto(nodes);
		return nodes;
	}
	
	/**
	 * Returns what kind of message this is
	 * @return An int value that can be either of the following:
	 * {@link #MESSAGE_DISCONNECT}, {@link #MESSAGE_FILE},
	 * {@link #MESSAGE_NODE_JOIN}, {@link #MESSAGE_NODE_LEFT}
	 * or {@link #MESSAGE_PRIMITIVES_OR_SERIALIZED_OBJECT}
	 */
	public int getMessageType() 
	{
		return messageType;
	}
	
	/**
	 * Returns a Node representation of the sender of this message
	 * @return The sender of this message
	 * @see peer2me.node.Node
	 */
	public Node getSender() 
	{
		return sender;
	}
	
	/**
	 * Sets the sender of this message. 
	 * 
	 * Should only be used by the framework itself. 
	 * @param sender The sender of this message
	 */
	public void setSender(Node sender) 
	{
		this.sender = sender;
	}
	
	/**
	 * Gets the ID of this message. A message may be split in several chunks in order to send it over the
	 * network. In such cases, all the chunks will have the same ID. 
	 * @return The ID of this message
	 */
	public String getMessageID()
	{
		return messageID;
	}
	
	/**
	 * Sets the ID of this message.
	 * 
	 * Should only be used internally in the framework
	 * @param messageID The ID that this message will be assosiated with
	 */
	public void setMessageID(String messageID)
	{
		this.messageID = messageID;
	}

	/**
	 * Adds a serialized object to this message.
	 * @param key The unique key that will be assosiated with the serialized object
	 * @param o A serialized object. The object MUST implement the interface {@link Serializable}.
	 * @throws InvalidKeyException Will be thrown if the key given is of lenght = 0 or null.
	 */
	public void addSerializedObject(String key, Object o) throws InvalidKeyException
	{
		if(key.length() == 0 || key == null)
			throw new InvalidKeyException();
		
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DataOutputStream dos = new DataOutputStream(baos);
		
		new ObjectOutputStream(o,dos);
		
		byte[] objectBytes = baos.toByteArray();
		MessagePart messagePart = new MessagePart(key,objectBytes);
		messagePart.setMessagePartType(MessagePart.SERIALIZEDOBJECT);
		messageParts.put(key,messagePart);
	}
	
	/**
	 * Gets the serialized object that is assosiated with the given key
	 * @param key The unique key that is assosiated with the object
	 * @return A serialized object which MUST be cast to the correct serialized object when returned
	 * @throws UnknownKeyException If the serialized object is not found
	 */
	public Object getSerializedObject(String key) throws UnknownKeyException
	{
		MessagePart part = (MessagePart)messageParts.get(key);

		if(part == null)
			throw new UnknownKeyException();
		
		byte[] bytes = part.getObjectBytes();
		ByteArrayInputStream bis = new ByteArrayInputStream(bytes);
		ObjectInputStream ois = new ObjectInputStream(new DataInputStream(bis));
		return ois.getObject();
	}
	
	/**
	 * Adds an array of bytes to this message as a messagePart. The array can only contain the bytes of a
	 * serialized object or primitive variables stored by the method <code>storeAllPrimitivesInOnePart</code>
	 * @param key The key can only be assosiated to a serialized object or primitive variables stored in an 
	 * array of bytes. The latter will always be assosiated with the key <code>primitives</code>.
	 * @param bytes The array of bytes that will be stored in this message as a MessagePart.
	 * @see peer2me.message.Message#storeAllPrimitivesInOnePart()
	 * @see peer2me.message.Message#addSerializedObject(String, Object)
	 */
	public void addObjectBytes(String key, byte[] bytes)
	{
		MessagePart part = new MessagePart(key,bytes);
		messageParts.put(key,part);
	}
	
	/**
	 * Sets the type of this message
	 * @param messageType Must be one of the following: {@link #MESSAGE_DISCONNECT}, {@link #MESSAGE_FILE}, 
	 * {@link #MESSAGE_NODE_JOIN}, {@link #MESSAGE_NODE_LEFT}, 
	 * or {@link #MESSAGE_PRIMITIVES_OR_SERIALIZED_OBJECT}
	 */
	public void setMessageType(int messageType)
	{
		this.messageType = messageType;
	}
}