package peer2me.message;

/**
 * There are three main types of information that can be stored in a MessagePart:
 * serialized objectBytes, fileObject and primitive variables such as int, double, String etc...
 * Each MessagePart can only contain ONE of these. The MessageParts are stored in a HashMap in Message.
 * 
 * All constants in this class must be in the range 400-499
 * @author Kim Saxlund
 *
 */
public class MessagePart
{
	private String key;
	private int messagePartType;
	
	//Not necessarily used
	private byte[] objectBytes;
	
	//Not necessarily used
	private FileObject fileObject;
	
	//Not necessarily used
	private FileInfo fileInfo;
	
	private int intValue;
	private double doubleValue;
	private float floatValue;
	private String stringValue;
	private boolean booleanValue;
	private char charValue;
	private long longValue;
	private short shortValue;
	private char[] chars;

	/**
	 * If this MessagePart contains a one or more primitive values (which is stored as an array of bytes}, 
	 * the method {@link #getMessagePartType()} will return this constant. 
	 */
	public static final int ALLPRIMITIVES = 400;	
	/**
	 * If this MessagePart contains a boolean, the method {@link #getMessagePartType()} will return this 
	 * constant
	 */
	public static final int BOOLEAN = 401;
	/**
	 * If this MessagePart contains a char, the method {@link #getMessagePartType()} will return this 
	 * constant
	 */
	public static final int CHAR = 402;
	/**
	 * If this MessagePart contains an array of chars, the method {@link #getMessagePartType()} will 
	 * return this constant
	 */
	public static final int CHARS = 403;
	/**
	 * If this MessagePart contains a double, the method {@link #getMessagePartType()} will return this 
	 * constant
	 */
	public static final int DOUBLE = 404;
	/**
	 * If this MessagePart contains a FileInfo, the method {@link #getMessagePartType()} will return this 
	 * constant
	 */
	public static final int FILEINFO = 405;
	/**
	 * If this MessagePart contains a FileObject, the method {@link #getMessagePartType()} will return this 
	 * constant
	 */
	public static final int FILEOBJECT = 406;
	/**
	 * If this MessagePart contains a float, the method {@link #getMessagePartType()} will return this 
	 * constant
	 */
	public static final int FLOAT = 407;
	/**
	 * If this MessagePart contains an int, the method {@link #getMessagePartType()} will return this 
	 * constant
	 */
	public static final int INT = 408;
	/**
	 * If this MessagePart contains a long, the method {@link #getMessagePartType()} will return this 
	 * constant
	 */
	public static final int LONG = 409;
	/**
	 * If this MessagePart contains a serialized object (which is stored as an array of bytes), 
	 * the method {@link #getMessagePartType()} will return this constant
	 */
	public static final int SERIALIZEDOBJECT = 410;
	/**
	 * If this MessagePart contains a short, the method {@link #getMessagePartType()} will return this 
	 * constant
	 */
	public static final int SHORT = 411;
	/**
	 * If this MessagePart contains a String, the method {@link #getMessagePartType()} will return this 
	 * constant
	 */
	public static final int STRING = 412;
	
	/**
	 * Creates a MessagePart that contains a FileInfo object
	 * @param key The key that identifies this MessagePart
	 * @param fileInfo Sets a FileInfo object as contents of this part
	 */
	public MessagePart(String key, FileInfo fileInfo)
	{
		this.key = key;
		this.fileInfo = fileInfo;
		messagePartType = FILEINFO;
	}
	
	/**
	 * Creates a MessagePart that contains a FileObject
	 * @param key The key that identifies this MessagePart
	 * @param fileObject Sets a FileObject object as contents of this part
	 */
	public MessagePart(String key, FileObject fileObject)
	{
		this.key = key;
		this.fileObject = fileObject;
		messagePartType = FILEOBJECT;
	}
	
	/**
	 * Creates a MessagePart that contains an int
	 * @param key The key that identifies this MessagePart
	 * @param intValue Sets an int as contents of this part
	 */
	public MessagePart(String key, int intValue)
	{
		this.key = key;
		this.intValue = intValue;
		messagePartType = INT;
	}
	
	/**
	 * Creates a MessagePart that contains a double
	 * @param key The key that identifies this MessagePart
	 * @param doubleValue Sets a double as contents of this part
	 */
	public MessagePart(String key, double doubleValue)
	{
		this.key = key;
		this.doubleValue = doubleValue;
		messagePartType = DOUBLE;
	}
	
	/**
	 * Creates a MessagePart that contains a float
	 * @param key The key that identifies this MessagePart
	 * @param floatValue Sets a float as contents of this part
	 */
	public MessagePart(String key, float floatValue)
	{
		this.key = key;
		this.floatValue = floatValue;
		messagePartType = FLOAT;
	}
	
	/**
	 * Creates a MessagePart that contains a short
	 * @param key The key that identifies this MessagePart
	 * @param shortValue Sets a short as contents of this part
	 */
	public MessagePart(String key, short shortValue)
	{
		this.key = key;
		this.shortValue = shortValue;
		messagePartType = SHORT;
	}
	
	/**
	 * Creates a MessagePart that contains a String
	 * @param key The key that identifies this MessagePart
	 * @param stringValue Sets a String as contents of this part
	 */
	public MessagePart(String key, String stringValue)
	{
		this.key = key;
		this.stringValue = stringValue;
		messagePartType = STRING;
	}
	
	/**
	 * Creates a MessagePart that contains a boolean
	 * @param key The key that identifies this MessagePart
	 * @param booleanValue Sets a boolean as contents of this part
	 */
	public MessagePart(String key, boolean booleanValue)
	{
		this.key = key;
		this.booleanValue = booleanValue;
		messagePartType = BOOLEAN;
	}
	
	/**
	 * Creates a MessagePart that contains a char
	 * @param key The key that identifies this MessagePart
	 * @param charValue Sets a char as contents of this part
	 */
	public MessagePart(String key, char charValue)
	{
		this.key = key;
		this.charValue = charValue;
		messagePartType = CHAR;
	}
	
	/**
	 * Creates a MessagePart that contains a long
	 * @param key The key that identifies this MessagePart
	 * @param longValue Sets a long as contents of this part
	 */
	public MessagePart(String key, long longValue)
	{
		this.key = key;
		this.longValue = longValue;
		messagePartType = LONG;
	}
	
	/**
	 * Creates a MessagePart that contains an array of chars
	 * @param key The key that identifies this MessagePart
	 * @param charsValue Sets a char as contents of this part
	 */
	public MessagePart(String key, char[] charsValue)
	{
		this.key = key;
		this.chars = charsValue;
		messagePartType = CHARS;
	}
	
	/**
	 * Creates a MessagePart that contains an array of bytes
	 * @param key The key that identifies this MessagePart
	 * @param objectBytes Sets a byte array as contents of this part
	 */
	public MessagePart(String key, byte[] objectBytes)
	{
		this.key = key;
		this.objectBytes = objectBytes;
		messagePartType = SERIALIZEDOBJECT;
	}
	
	/**
	 * Sets the key of this MessagePart
	 * @param key The key that identifies this MessagePart
	 */
	public void setKey(String key)
	{
		this.key = key;
	}
	
	/**
	 * Returns the key of this MessagePart
	 * @return The key of this MessagePart
	 */
	public String getKey()
	{
		return key;
	}

	/**
	 * A call to this method persumes that there exists an array of bytes in this MessagePart.<br>
	 * Returns an array of bytes that is attached to this MessagePart
	 * @return An array of bytes
	 */
	public byte[] getObjectBytes() 
	{
		return objectBytes;
	}

	/**
	 * Sets an array of bytes as contents of this MessagePart
	 * @param objectBytes The array of bytes. This is either the bytes of a serialized object, or primitive
	 * value(s) stored in an array of bytes.
	 */
	public void setObjectBytes(byte[] objectBytes) 
	{
		this.objectBytes = objectBytes;
	}

	/**
	 * Returns what kind of MessagePart this is. The values returned are one of the following: 
	 * {@link #ALLPRIMITIVES}, {@link #BOOLEAN}, {@link #CHARS}, {@link #DOUBLE}, {@link #FILEINFO},
	 * {@link #FILEOBJECT}, {@link #FLOAT}, {@link #INT}, {@link #LONG}, {@link #SHORT}, {@link #SERIALIZEDOBJECT}
	 * or {@link #STRING}. 
	 * @return An constant that identifies this MessagePart
	 */
	public int getMessagePartType()
	{
		return messagePartType;
	}

	/**
	 * Sets the type of this MessagePart.
	 * @param messagePartType Must be one of the following: {@link #ALLPRIMITIVES}, {@link #BOOLEAN}, 
	 * {@link #CHARS}, {@link #DOUBLE}, {@link #FILEINFO}, {@link #FILEOBJECT}, {@link #FLOAT}, 
	 * {@link #INT}, {@link #LONG}, {@link #SHORT}, {@link #SERIALIZEDOBJECT} or {@link #STRING}. 
	 */
	public void setMessagePartType(int messagePartType)
	{
		this.messagePartType = messagePartType;
	}

	/**
	 * A call to this method persumes that there exists a boolean in this MessagePart.<br>
	 * Returns a boolean that is attached to this MessagePart
	 * @return The boolean attached to this MessagePart
	 */
	public boolean getBooleanValue()
	{
		return booleanValue;
	}

	/**
	 * A call to this method persumes that there exists an array of chars in this MessagePart.<br>
	 * Returns an array of chars that is attached to this MessagePart
	 * @return The array of chars attached to this MessagePart
	 */
	public char[] getChars()
	{
		return chars;
	}

	/**
	 * A call to this method persumes that there exists a char in this MessagePart.<br>
	 * Returns a char that is attached to this MessagePart
	 * @return The char attached to this MessagePart
	 */
	public char getCharValue()
	{
		return charValue;
	}

	/**
	 * A call to this method persumes that there exists a double in this MessagePart.<br>
	 * Returns a double that is attached to this MessagePart
	 * @return The double attached to this MessagePart
	 */
	public double getDoubleValue()
	{
		return doubleValue;
	}

	/**
	 * A call to this method persumes that there exists a float in this MessagePart.<br>
	 * Returns a float that is attached to this MessagePart
	 * @return The float attached to this MessagePart
	 */
	public float getFloatValue()
	{
		return floatValue;
	}

	/**
	 * A call to this method persumes that there exists a int in this MessagePart.<br>
	 * Returns a int that is attached to this MessagePart
	 * @return The int attached to this MessagePart
	 */
	public int getIntValue()
	{
		return intValue;
	}

	/**
	 * A call to this method persumes that there exists a long in this MessagePart.<br>
	 * Returns a long that is attached to this MessagePart
	 * @return The long attached to this MessagePart
	 */
	public long getLongValue()
	{
		return longValue;
	}

	/**
	 * A call to this method persumes that there exists a short in this MessagePart.<br>
	 * Returns a short that is attached to this MessagePart
	 * @return The short attached to this MessagePart
	 */
	public short getShortValue()
	{
		return shortValue;
	}

	/**
	 * A call to this method persumes that there exists a String in this MessagePart.<br>
	 * Returns a String that is attached to this MessagePart
	 * @return The String attached to this MessagePart
	 */
	public String getStringValue()
	{
		return stringValue;
	}

	/**
	 * A call to this method persumes that there exists a FileObject in this MessagePart.<br>
	 * Returns a FileObject that is attached to this MessagePart
	 * @return The FileObject attached to this MessagePart
	 */
	public FileObject getFileObject()
	{
		return fileObject;
	}

	/**
	 * A call to this method persumes that there exists a FileInfo in this MessagePart.<br>
	 * Returns a FileInfo that is attached to this MessagePart
	 * @return The FileInfo attached to this MessagePart
	 */
	public FileInfo getFileInfo()
	{
		return fileInfo;
	}
}