package peer2me.message;

import java.io.IOException;
import java.io.OutputStream;

import javax.microedition.io.Connector;
import javax.microedition.io.file.FileConnection;

import peer2me.log.Log;
import peer2me.log.LogElementType;
import peer2me.util.FileHandler;

/**
 * This class extends {@link FileObject} and implements {@link java.lang.Runnable} and is used by the 
 * {@link peer2me.message.Message} class. When the framework receives a file or parts of 
 * a file from the network, an instance of this class is created and added to the Message in a
 * {@link peer2me.message.MessagePart}. Bytes are then constantly written to a FileOutPutStream in this class
 * when new parts of the file is received. 
 * 
 * <b>An instance of this class should only be created by the framework itself. </b>
 * @author Kim Saxlund
 *
 */
public class ReceivedFileObject extends FileObject implements Runnable
{
	private OutputStream out;
	
	/**
	 * Constructor that starts a thread when called. The thread is started to avoid blocking of the Midlet.
	 * The thread creates a connection to the filesystem, which is a operation which must be done in a 
	 * separate thread than the Midlet itself. 
	 * @param fileName The name of the file that is created
	 * @param fileSize The total size of the file that is created
	 */
	public ReceivedFileObject(String fileName, long fileSize)
	{
		super(FileHandler.getInstance().getReceivedFilesFolder() + fileName);
		this.fileSize = fileSize;
		this.fileName = fileName;
		Thread thread = new Thread(this);
		thread.start();
	}

	/**
	 * Creates a directory to store the file in if the directory does not exists. By default, the directory
	 * created is named <code>tmpp2me</code> and is created in one of the root directories of the mobile phone.
	 */
	public void run()
	{
		try 
		{
			FileConnection newDir = null;
			newDir = (FileConnection) Connector.open(FileHandler.getInstance().getReceivedFilesFolder());
			
			if(!newDir.exists())
			{
				newDir.mkdir();				
				Log.getInstance().addElement(LogElementType.INFORMATION,"Created: " + FileHandler.getInstance().getReceivedFilesFolder());
			}
			newDir.close();			
			fileConnection.create();			
			out = fileConnection.openOutputStream();
			
			accessGranted = true;
		} 
		catch (IOException e) 
		{
			Log.getInstance().addElement(LogElementType.ERROR,"RFO:"+e.getMessage());
		}		
	}
	
	/**
	 * Appends a number of bytes to the file assosiated with this class
	 * @param bytes The bytes that are appended to the file
	 */
	public void write(byte[] bytes)
	{
		if(out == null)
		{
			try 
			{
				out = fileConnection.openOutputStream();
			} 
			catch (IOException e1) 
			{
				Log.getInstance().addElement(LogElementType.ERROR,"RFO:"+e1.getMessage());
			}
		}
		try
		{
			out.write(bytes);
		} 
		catch (IOException e)
		{
			Log.getInstance().addElement(LogElementType.ERROR, "RFO: " + e.getMessage());
		}
	}
	
	/**
	 * When there are no more bytes to write, this method closes the streams used.
	 *
	 */
	public void finishedWriting()
	{
		try
		{
			out.close();
			fileConnection.close();
		} 
		catch (IOException e)
		{
			Log.getInstance().addElement(LogElementType.ERROR,"Could not close:"+e.getMessage());
		}
	}
	
	/**
	 * Sets the name of the file assosiated with this class
	 * @param fileName The name of the file
	 */
	public void setFileName(String fileName)
	{
		this.fileName = fileName;
	}
}
