package peer2me.message;

import java.io.IOException;
import java.io.InputStream;

import peer2me.exception.FileNotFoundException;
import peer2me.log.Log;
import peer2me.log.LogElementType;

/**
 * This class extends {@link FileObject} and implements {@link java.lang.Runnable} and is used by the 
 * {@link peer2me.message.Message} class. When a file is attached to a Message that is to be sent, a 
 * {@link peer2me.message.MessagePart} with this SendFileObject class is added to the Message. 
 * This class is capable of reading a file from the filesystem in chunks. 
 * @author Kim Saxlund
 *
 */
public class SendFileObject extends FileObject implements Runnable
{
	private InputStream fis;
	private final int chunkSize = 50000;
	private long bytesLeft;
	private int numberOfBytesJustRead;
	
	/**
	 * Constructor that starts a thread when called. The thread is started to avoid blocking of the Midlet.
	 * The thread creates a connection to the filesystem, which is a operation which must be done in a 
	 * separate thread than the Midlet itself. 
	 * @param fullPathToFile The full path to the file
	 */
	public SendFileObject(String fullPathToFile)
	{
		super("file:///" + fullPathToFile);
		this.fileName = fullPathToFile.substring(fullPathToFile.lastIndexOf('/')+1,fullPathToFile.length());
		Thread thread = new Thread(this);
		thread.start();
	}

	/**
	 * Retrives information about the size of the file and sets private variables. Also opens an 
	 * FileInputStream to the file.
	 */
	public void run()
	{
		if (!fileConnection.exists())
		{
			Log.getInstance().addElement(LogElementType.ERROR,"File does not exist");
			throw new FileNotFoundException();
		}
		else
		{
			try
			{
				fileSize = fileConnection.fileSize();
				fis = fileConnection.openInputStream();
				accessGranted = true;
				bytesLeft = fileSize;
			} 
			catch (IOException e)
			{
				Log.getInstance().addElement(LogElementType.ERROR,"SendFileObject_Run:"+e.getMessage());
			}
		}
	}
	
	/**
	 * Reads a chunk of a file and returns the data in an array. The chunk is determined by the 
	 * private variable <code>chunkSize</code>. This method continues to read from the file form where it last
	 * stopped. If the end of the file is reached, the fileconnection and
	 * fileinputstream is closed and the last bytes are returned.
	 * @return A chunk of the file in a byte array. 
	 * variable <code>chunkSize</code>
	 */
	public byte[] read()
	{
		try
		{
			int arraySize;
			if(bytesLeft < chunkSize)
				arraySize = (int)bytesLeft;
			else
				arraySize = (int)chunkSize;
			byte[] b = new byte[arraySize];
			
			numberOfBytesJustRead = fis.read(b,0,arraySize);
			
			bytesLeft = bytesLeft - arraySize;
			if(bytesLeft == 0)
			{
				fileConnection.close();
				fis.close();
			}
			return b;			
		}
		catch(IOException ioe)
		{
			Log.getInstance().addElement(LogElementType.ERROR,"SendFileObject_read():"+ioe.getMessage());
			return null;
		}
	}
	
	/**
	 * Calculates and returns how many chunks the file must be read in. For instance, if
	 * this method returns <code>20</code>, the method {@link #read()} must be called 20 times before the
	 * entire file has been read.
	 * @return The number of times the {@link #read()} method must be called before the entire file has been read
	 */
	public int numberOfChunksNeeded()
	{
		if(fileSize < chunkSize)
		{
			return 1;
		}
		double tmp = (double)fileSize / (double)chunkSize;
		int tmp2 = (int)Math.ceil(tmp);
		return tmp2;
	}
	
	/**
	 * Returns the number of bytes that has not be read from the file yet
	 * @return The number of bytes that remains to be read from the file
	 */
	public long getBytesLeft()
	{
		return bytesLeft;
	}

	/**
	 * Returns how many bytes was read during the last call to {@link #read()}
	 * @return How many bytes was last read
	 */
	public int getNumberOfBytesJustRead()
	{
		return numberOfBytesJustRead;
	}	
}
