package peer2me.message;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Stack;

/**
 * This class implements the interface Serializable and is used by the framework to send information about 
 * all the nodes in the network. This class is used as a linked list, since it also contains an instance of
 * itself. This enables the framework to iterate through the class easily. SendableNodeInfo is used by the 
 * classes {@link peer2me.network.bluetooth.BluetoothListener} and {@link peer2me.network.bluetooth.BluetoothSearcher}
 * 
 * <b>Should only be used by the framework itself</b>
 * 
 * @author saxlund
 * @see Serializable
 *
 */
public class SendableNodeInfo implements Serializable
{
	String nodeName = "";
	String nodeAddress = "";
	String connectionURL = "";
	String pingConnectionUrl = "";
	SendableNodeInfo nextNode;
	
	/**
	 * Required default constructor so that the class {@link ObjectInputStream} can instanciate the class
	 *
	 */
	public SendableNodeInfo()
	{}
	
	/**
	 * Constructor
	 * @param nodeName The name of the node
	 * @param nodeAddress The address of the node
	 * @param connectionURL The URL which another node has to use in order to connect to the given node
	 * @param pingConnectionURL The URL which another node has to use in order to ping the given node
	 */
	public SendableNodeInfo(String nodeName, String nodeAddress, String connectionURL, String pingConnectionURL)
	{
		this.nodeName = nodeName;
		this.nodeAddress = nodeAddress;
		this.connectionURL = connectionURL;
		this.pingConnectionUrl = pingConnectionURL;
	}

	/**
	 * Returns the connectionURL of the node in this class
	 * @return The connectionURL of the node in this class
	 */
	public String getConnectionURL()
	{
		return connectionURL;
	}

	/**
	 * Returns the address to this node
	 * @return The address to this node
	 */
	public String getNodeAddress()
	{
		return nodeAddress;
	}

	/**
	 * Returns the name to this node
	 * @return The name to this node
	 */
	public String getNodeName()
	{
		return nodeName;
	}

	/**
	 * Method required by the interface {@link Serializable}. Deserializes the datainputstream to a
	 * SendableNodeInfo object.
	 */
	public void deSerialize(DataInputStream dis, Stack stack) throws IOException, ClassNotFoundException, IllegalAccessException, InstantiationException
	{
		String name = dis.readUTF();
		this.setNodeName(name);
		this.setNodeAddress(dis.readUTF());
		this.setConnectionURL(dis.readUTF());
		this.setPingConnectionUrl(dis.readUTF());
		if(0x0 != dis.readByte())
		{
			this.nextNode =	(peer2me.message.SendableNodeInfo)Class.forName("peer2me.message.SendableNodeInfo").newInstance();
			stack.push(this.nextNode);
		}
	}
	
	/**
	 * Method required by the interface {@link Serializable}. Before an instance of this class can be
	 * sent over the network, it must first be serialized. This method writes the content of this class
	 * to the dataoutputstream given in the first parameter of this method.
	 */
	public void serialize(DataOutputStream dos, Stack stack) throws IOException
	{
		dos.writeUTF(this.nodeName);
		dos.writeUTF(this.nodeAddress);
		dos.writeUTF(this.connectionURL);
		dos.writeUTF(this.pingConnectionUrl);
		if(nextNode == null)
		{
			dos.writeByte((byte)0x0);
		}
		else
		{
			dos.writeByte((byte)0x7F);
			stack.push(this.nextNode);
		}		
	}

	/**
	 * Returns the next node in the linked list if there is one
	 * @return The next node. If the next node is null, then null is returned
	 */
	public SendableNodeInfo getNextNode()
	{
		return this.nextNode;
	}

	/**
	 * Sets the next node in the linked list
	 * @param nextNode The node that will be the next node in the list
	 */
	public void setNextNode(SendableNodeInfo nextNode)
	{
		this.nextNode = nextNode;
	}

	/**
	 * Sets the connectionURL of the node in this class
	 * @param connectionURL The URL that must be used to connect to the node in this class
	 */
	public void setConnectionURL(String connectionURL)
	{
		this.connectionURL = connectionURL;
	}

	/**
	 * Sets the address of this node
	 * @param nodeAddress The address of this node
	 */
	public void setNodeAddress(String nodeAddress)
	{
		this.nodeAddress = nodeAddress;
	}

	/**
	 * Sets the name of this node
	 * @param nodeName The name of this node
	 */
	public void setNodeName(String nodeName)
	{
		this.nodeName = nodeName;
	}

	/**
	 * Returns the pingConnectionURL of the node in this class
	 * @return The pingConnectionURL of the node in this class
	 */
	public String getPingConnectionUrl()
	{
		return pingConnectionUrl;
	}

	/**
	 * Sets the pingConnectionURL of the node in this class
	 * @param pingConnectionUrl of the node in this class
	 */
	public void setPingConnectionUrl(String pingConnectionUrl)
	{
		this.pingConnectionUrl = pingConnectionUrl;
	}
	
}
