package peer2me.network;

import peer2me.message.Message;
import peer2me.node.Node;

/**
 * This is an abstract class representing the
 * network used for communication. Should be extended
 * by the class representing networktechnology.
 * I.e. {@link peer2me.network.bluetooth.Bluetooth} extends this class.
 * @author Tommy Bjoernsgaard
 *
 */
public abstract class Network
{
	/**
	 * Default empty constructor
	 *
	 */
	public Network()
	{
	}
	
	/**
	 * This method should initialize the framework making
	 * it discoverable and ready to perform a search
	 *
	 */
	public abstract void initialize();
	
	/**
	 * This method should perform a search for services and devices
	 * @return true if a search can be performed, false if it is not possible
	 */
	public abstract boolean search();
	
	/**
	 * This method should send a {@link peer2me.message.Message} to another
	 * {@link peer2me.node.Node}
	 * @param message the Message to send
	 */
	public abstract void sendMessage(Message message);

	
	/**
	 * This method sets if the ping-functionality should be used or not
	 * @param activeatePing true if the ping should be used, else false
	 */
	public abstract void setActivatePing(boolean activeatePing);
	
	/**
	 * This method should be called whenever a new node is found.
	 * @param networkNode the networkNode that is found
	 */
	public abstract void nodeFound(NetworkNode networkNode);
	
	/**
	 * This method should be called whenever a node is lost or
	 * abscent.
	 * @param node the node that is lost.
	 */
	public abstract void nodeLost(Node node);
	
	/**
	 * This method should be called whenever a {@link peer2me.message.Message} is received
	 * @param message the message that is received
	 */
	public abstract void messageReceived(Message message);
	

	/**
	 * This method is needed by {@link peer2me.network.NetworkTimer} to alert whenever a ping
	 * should run
	 *
	 */
	public abstract void ping();
	
	/**
	 * This method should run {@link peer2me.node.Node#incrementEcho()} on the
	 * given {@link peer2me.node.Node} 
	 * @param node the node that has responded
	 */
	public abstract void echo(Node node);
	
	/**
	 * This method should clean up the framework, closing active connections etc.
	 * Is intended to be used when exiting the framework/application
	 *
	 */
	public abstract void clean();
	
	/**
	 * This method should be called wheneve a search is finished.
	 *
	 */
	public abstract void searchCompleted();
}
