package peer2me.network.bluetooth;

import peer2me.log.Log;
import peer2me.log.LogElementType;
import peer2me.message.LinkedMessageList;
import peer2me.message.LinkedPingRecipientsList;
import peer2me.message.Message;
import peer2me.network.Network;
import peer2me.network.NetworkNode;
import peer2me.network.NetworkTimer;
import peer2me.node.Node;
import peer2me.service.Service;
import peer2me.session.Session;

/**
 * This class extends the {@link peer2me.network.Network} and every Bluetooth network operation is
 * controlled indirectly from this class. This class starts two listeners that are able to establish
 * incomming connections. It also have two queues that are used for outgoing messages and pings.
 * @author Kim Saxlund
 *
 */
public class Bluetooth extends Network
{
	private BluetoothSearcher bluetoothSearcher;
	private LinkedMessageList messageQue;
	private LinkedPingRecipientsList pingQue;	
	private MessageQueueProcessor messageQueProcessor;
	private PingQueueProcessor pingMessageQueProcessor;	
	private BluetoothListener bluetoothListener;
	private BluetoothPingListener bluetoothPingListener;	
	private NetworkTimer networkTimer;
	
	private Service service;
	private String localConnectionURL;
	private String localPingConnectionURL;
	private long PINGRATE = 20000;
	private int pingID = 1;	
	private boolean goodbyeSent = true;
	private boolean activatePing;
	/**
	 * Constructor that creates two empty queues for outgoing messages and pings
	 *
	 */
	public Bluetooth()
	{
		super();
		messageQue = new LinkedMessageList();
		pingQue = new LinkedPingRecipientsList();
	}
	
	/**
	 * Starts accepting incoming connections and starts the queue processors
	 */
	public void initialize()
	{
		service = Session.getInstance().getService();
		bluetoothListener = new BluetoothListener(this,service);
		bluetoothPingListener = new BluetoothPingListener(this,service);
		messageQueProcessor = new MessageQueueProcessor(this);
		pingMessageQueProcessor = new PingQueueProcessor(this);
	}
	
	/**
	 * Starts a search for other devices if there are no more messages in the queue. This is because
	 * Bluetooth cannot perform a search and send messages at the same time
	 * @return True if a search was started, false otherwise
	 */
	public boolean search()
	{		
		if(messageQue.getNumberOfMessagesInList() > 0)
			return false;
		bluetoothSearcher = new BluetoothSearcher(this,service);
		bluetoothSearcher.initialize();
		bluetoothSearcher.search();
		activatePing();
		return true;
	}
	
	public void echo(Node node)
	{	
	}

	/**
	 * Puts a message in the queue. The {@link MessageQueueProcessor} takes care of processing the message.
	 * @param message The message that is put in the queue
	 */
	public void sendMessage(Message message)
	{
		messageQue.insertLast(message);
	}
	
	/**
	 * When the {@link BluetoothConnectionHandler} receives a message, it reconstructs it and invokes
	 * this method when the message is reconstructed. This method forwards the message to {@link Session}.
	 * @param message
	 */
	public void messageReceived(Message message) 
	{		
		Session.getInstance().messageReceived(message);
	}
	
	/**
	 * Every time when the {@link BluetoothConnectionHandler} receives a part of a message, it invokes
	 * this method. This method forwards the part number and total parts to {@link Session}.
	 * @param part The part number that was received
	 * @param total Total number of parts in the message
	 */
	public void messagePartReceived(String messageID, int part, int total)
	{
		Session.getInstance().messagePartReceived(messageID,part,total);
	}

	/**
	 * When the {@link BluetoothSearcher} has found other nodes, it invokes this method. The method is also
	 * invoked if the {@link BluetoothConnectionHandler} receives a message that contains information
	 * about other nodes. 
	 */
	public void nodeFound(NetworkNode networkNode)
	{
		Session.getInstance().nodeFound(networkNode);
		goodbyeSent = false;
		activatePing();
	}
	
	/**
	 * Is invoked by {@link BluetoothSearcher} when a search has completed. No mather what the result of the
	 * search was, this method will be called either way. This is so that the application knows when a 
	 * search is completed. A usual scenario might just be that no nodes where found.
	 */
	public void searchCompleted()
	{
		Session.getInstance().searchCompleted();
	}
		
	/**
	 * If a node leaves the network or does not respond to pings, this method is invoked.  
	 * @param node The node that is lost
	 */
	public void nodeLost(Node node)
	{
		Session.getInstance().nodeLost(node);
	}
	
	/**
	 * This method puts all nodes in the pingQueue so that the {@link PingQueueProcessor} can process
	 * the pings. This method is invoked at regular intervals by the {@link NetworkTimer}.
	 */
	public void ping()
	{
		Node[] nodes = Session.getInstance().getAllNodes();
		pingID++;
		for (int i = 0; i < nodes.length; i++)
		{
			BluetoothNode bluetoothNode = (BluetoothNode)nodes[i].getNetworkNode();
			if(bluetoothNode.getPingConnectionURL() != "" && bluetoothNode.getAddress() != (Session.getInstance().getLocalNode().getAddress()))
				pingQue.insert(nodes[i]);
		}		
	}
	
	/**
	 * Checks to see if nodes has responded to pings. If they have not, they are reported as lost.
	 *
	 */
	public void checkPingResponse(Node node)
	{
		if(node.isNodeAbscent())
			nodeLost(node);	
	}
	
	/**
	 * Sets a local variable determining if ping shall be used
	 * @param activatePing True if ping, false otherwise
	 */
	public void setActivatePing(boolean activatePing)
	{
		this.activatePing = activatePing;
	}
	
	/**
	 * Activates the ping functionality. This method is invoked when a node is found
	 *
	 */
	public void activatePing()
	{
		if(activatePing)
		{
			networkTimer = new NetworkTimer(this,PINGRATE);
			Log.getInstance().addElement(LogElementType.INFORMATION,"Ping activated");
		}
	}
	
	/**
	 * Stops the listeners from reveiving connections. Must be called by the application through 
	 * {@link peer2me.framework.Framework#clean()} upon exit.
	 */
	public void clean()
	{
		bluetoothListener.toString();
		bluetoothListener = null;
		bluetoothPingListener.toString();
		bluetoothPingListener = null;
		if(networkTimer != null)
		{
			networkTimer.cancel();
			networkTimer = null;
		}
		Message message = new Message();
		message.setMessageType(Message.MESSAGE_NODE_LEFT);
		message.addElement("left","bye");
		message.addRecipients(Session.getInstance().getAllNodes());
		sendMessage(message);
		//Does not exit until message has been sent
		while(!goodbyeSent)
		{
			try
			{
				Thread.sleep(100);
			} catch (InterruptedException e)
			{
				e.printStackTrace();
			}
		}
		messageQueProcessor.isProcessQueue();
		messageQueProcessor = null;
		pingMessageQueProcessor.isProcessQue();
		pingMessageQueProcessor = null;
	}

	/**
	 * Returns the connectionURL of the local node
	 * @return The connectionURL of the local node
	 */
	public String getLocalConnectionURL() 
	{
		return localConnectionURL;
	}

	/**
	 * Sets the connectionURL of the local node
	 * @param localConnectionURL The connectionURL of the local node
	 */
	public void setLocalConnectionURL(String localConnectionURL) 
	{
		this.localConnectionURL = localConnectionURL;
	}

	/**
	 * Returns the list that contains all messages that are to be sent
	 * @return The queue, represented as a {@link LinkedMessageList}
	 */
	public LinkedMessageList getMessageQue()
	{
		return messageQue;
	}

	/**
	 * Returns the list that contains all pings that are to be sent
	 * @return The queue, represented as a {@link LinkedPingRecipientsList}
	 */
	public LinkedPingRecipientsList getPingQue()
	{
		return pingQue;
	}

	/**
	 * Returns the connectionURL that must be used to ping the local node
	 * @return The pingConnectionURL
	 */
	public String getLocalPingConnectionURL()
	{
		return localPingConnectionURL;
	}

	/**
	 * Sets the connectionURL that must be used to ping the local node
	 * @param localPingConnectionURL The URL that is set as the localPingConnectionURL
	 */
	public void setLocalPingConnectionURL(String localPingConnectionURL)
	{
		this.localPingConnectionURL = localPingConnectionURL;
	}

	/**
	 * To allow the node to exit, this method must be used
	 * @param canILeaveNow True if the node can exit, false otherwise
	 */
	public void setGoodbyeSent(boolean canILeaveNow)
	{
		this.goodbyeSent = canILeaveNow;
	}
}
