package peer2me.network.bluetooth;

import java.io.IOException;
import java.io.InputStream;

import javax.microedition.io.Connection;
import javax.obex.HeaderSet;
import javax.obex.Operation;
import javax.obex.ResponseCodes;
import javax.obex.ServerRequestHandler;

import peer2me.exception.GroupNotFoundException;
import peer2me.group.Group;
import peer2me.log.Log;
import peer2me.log.LogElementType;
import peer2me.message.FileInfo;
import peer2me.message.Message;
import peer2me.message.MessagePart;
import peer2me.message.ReceivedFileObject;
import peer2me.message.SendableNodeInfo;
import peer2me.node.Node;
import peer2me.session.Session;

/**
 * This class is responsible for handling all incomming connections and messages except for pings.
 * This class extends <a href="http://www.j2medev.com/api/btapi/javax/obex/ServerRequestHandler.html">javax.obex.ServerRequestHandler</a>
 * @author Kim Saxlund
 */
public class BluetoothConnectionHandler extends ServerRequestHandler
{
	private Connection connection;
	private BluetoothListener bluetoothListener;
	private Bluetooth bluetooth;
	
	private boolean finished = false;	
	private byte[] readBytes;
	
	/**
	 * Constructor that sets local references to the parameters given
	 * @param connection
	 * @param bluetoothListener
	 * @param bluetooth
	 */
	public BluetoothConnectionHandler(Connection connection, BluetoothListener bluetoothListener, Bluetooth bluetooth)
	{
		this.connection = connection;
		this.bluetoothListener = bluetoothListener;
		this.bluetooth = bluetooth;
	}
	
	/**
	 * Inherited method from javax.obex.ServerRequestHandler. 
	 * Is invoked when a remote device tries to
	 * establish a connection. This implementation accepts all connections.
	 * @param reply A HeaderSet that can be used to send a reply
	 * @param request The HeaderSet that is sent from the remote device
	 */
	public int onConnect(HeaderSet request, HeaderSet reply) 
	{
		return ResponseCodes.OBEX_HTTP_OK;
	}
	
	/**
	 * Inherited method from javax.obex.ServerRequestHandler.
	 * This method is invoked whenever the application receives an incoming message. This method is quite 
	 * large because there are several types of messages that exists in the  Peer2Me framework.
	 * The messages can be of the types that are definde as public variables in {@link Message}.
	 * @param operation Contains HeaderSet and bytestream
	 */
	public int onPut (Operation operation) 
	{
		InputStream inputStream = null;

			String headerName = null;
			long messageLength = -1;
			String senderAdress = null;
			String messageID = null;
			int totalNumberOfMessages = -1;
			int messageNumber = -1;
			int type = -1;
			Message message = null;;
			try
			{
				inputStream = operation.openInputStream();
				headerName = (String)operation.getReceivedHeaders().getHeader(Peer2MeHeaderSet.NAME);
				messageLength = Long.parseLong((String)operation.getReceivedHeaders().getHeader(Peer2MeHeaderSet.LENGTH));
				senderAdress = (String)operation.getReceivedHeaders().getHeader(Peer2MeHeaderSet.SENDER_ADDRESS);
				messageID = (String)operation.getReceivedHeaders().getHeader(Peer2MeHeaderSet.MESSAGE_ID);	
				totalNumberOfMessages = Integer.parseInt((String)operation.getReceivedHeaders().getHeader(Peer2MeHeaderSet.TOTAL_NUMBER_OF_MESSAGES));
				messageNumber = Integer.parseInt((String)operation.getReceivedHeaders().getHeader(Peer2MeHeaderSet.MESSAGE_NUMBER));			
				type = Integer.parseInt((String)operation.getReceivedHeaders().getHeader(Peer2MeHeaderSet.TYPE));
			} 
			catch (NumberFormatException e1)
			{
				Log.getInstance().addElement(LogElementType.ERROR,"Number format exception" + e1.getMessage());
				e1.printStackTrace();
			} 
			catch (IOException e1)
			{
				Log.getInstance().addElement(LogElementType.ERROR,"IOException: " + e1.getMessage());
				e1.printStackTrace();
			}			

			message = (Message)bluetoothListener.getIncompleteReceivedMessages().get(messageID);
			boolean newmessage = false;
					
			byte[] bytesFromStream = readFromStream(inputStream,(int)messageLength);
			bluetooth.messagePartReceived(messageID, messageNumber,totalNumberOfMessages);
			switch(type)
			{
			case Message.MESSAGE_NODE_JOIN:					
				message = new Message();
				message.addObjectBytes(headerName, bytesFromStream);
				SendableNodeInfo linkedNodes = (SendableNodeInfo)message.getSerializedObject(headerName);
				addNewNodesToDefaultGroup(linkedNodes);
				bluetooth.activatePing();
				break;
			case Message.MESSAGE_NODE_LEFT:			
				Log.getInstance().addElement(LogElementType.INFORMATION,"NODE_LEFT:" + senderAdress);
				Node lostNode = Session.getInstance().getNode(senderAdress);
				bluetooth.nodeLost(lostNode);
				break;
			case Message.MESSAGE_FILE:
				String messagePartID = null;
				String fileName = null;
				String endOfFile = null;
				long fileSize = -1;
				try
				{
					messagePartID = (String)operation.getReceivedHeaders().getHeader(Peer2MeHeaderSet.MESSAGE_PART_ID);
					fileName = (String)operation.getReceivedHeaders().getHeader(Peer2MeHeaderSet.FILE_NAME);
					endOfFile = (String)operation.getReceivedHeaders().getHeader(Peer2MeHeaderSet.END_OF_FILE);
					fileSize = Long.parseLong((String)operation.getReceivedHeaders().getHeader(Peer2MeHeaderSet.FILE_SIZE));
				} 
				catch (NumberFormatException e1)
				{
					e1.printStackTrace();
				} 
				catch (IOException e1)
				{
					e1.printStackTrace();
				}
				if(message==null)
				{	
					Node sender = Session.getInstance().getNode(senderAdress);
					message = new Message();
					message.setSender(sender);
					newmessage = true;
				}
				MessagePart part = message.getMessagePart(messagePartID);
				if(part == null)
				{
					ReceivedFileObject fileObject = new ReceivedFileObject(fileName, fileSize);
					
					while(!fileObject.isAccessGranted())
					{
						try
						{
							Thread.sleep(100);
						} 
						catch (InterruptedException e)
						{
							e.printStackTrace();
						}
					}
					
					message.addReceivedFile(messagePartID,fileObject);
					part = message.getMessagePart(messagePartID);
				}
				
				ReceivedFileObject obj = (ReceivedFileObject)part.getFileObject();
				obj.write(bytesFromStream);
				if(endOfFile.equals("Yes"))
				{		
					Log.getInstance().addElement(LogElementType.INFORMATION, "End of file received");
					obj.finishedWriting();
					FileInfo fileInfo = new FileInfo(obj.getFileName(),obj.getFullPathToFile(),obj.getFileSize());
					message.addFileInfo(messagePartID,fileInfo);
				}
				if(newmessage)
					bluetoothListener.getIncompleteReceivedMessages().put(messageID,message);
				if(messageNumber == totalNumberOfMessages)
				{	
					message.extractPrimitivesToSeparateParts();
					bluetooth.messageReceived(message);
				}
				break;
			case Message.MESSAGE_NODE_MOVE:
				if(message==null)
				{	
					//The sender-node must be in the group in order for this to work
					Node sender = Session.getInstance().getNode(senderAdress);
					message = new Message();
					if(sender != null)
						message.setSender(sender);
					newmessage = true;
				}
				message.addObjectBytes(headerName, bytesFromStream);				
				
				if(newmessage)
					bluetoothListener.getIncompleteReceivedMessages().put(messageID,message);
				
				boolean ok = true;
				message.extractPrimitivesToSeparateParts();
				
				Group from = null;
				Group to = null;
				
				Node node = Session.getInstance().getNode(message.getString("node"));
				try
				{
					from = Session.getInstance().getGroup(message.getString("from"));
				} 
				catch (Exception e)
				{
					ok = false;
				}
				if(ok)
				{
					try
					{
						to = Session.getInstance().getGroup(message.getString("to"));
					} 
					catch (GroupNotFoundException e)
					{
						to = Session.getInstance().createGroup(message.getString("to"));
					}
					from.removeNode(node);
					to.addNode(node);
				}
				else
				{
					Log.getInstance().addElement(LogElementType.WARNING,"Could not move node!!");
				}

				break;
			case Message.MESSAGE_PRIMITIVES_OR_SERIALIZED_OBJECT:				
				if(message==null)
				{	
					//The sender-node must be in the group in order for this to work
					Node sender = Session.getInstance().getNode(senderAdress);
					message = new Message();
					if(sender != null)
						message.setSender(sender);
					newmessage = true;
				}
				message.addObjectBytes(headerName, bytesFromStream);				
				
				if(newmessage)
					bluetoothListener.getIncompleteReceivedMessages().put(messageID,message);
				
				if(messageNumber == totalNumberOfMessages)
				{	
					message.extractPrimitivesToSeparateParts();
					bluetooth.messageReceived(message);
				}
				break;
			default:
				break;
			}
		 
		try 
		{
			connection.toString();
			operation.close();
		} 
		catch (IOException e) 
		{
			Log.getInstance().addElement(LogElementType.ERROR,"Error closing: " + e.getMessage());
			e.printStackTrace();
		}
		if(messageNumber == totalNumberOfMessages)
		{
			finished = true;
			synchronized (bluetoothListener.getThread()) 
			{
				bluetoothListener.getThread().notify();
			}
		}
		return ResponseCodes.OBEX_HTTP_OK;
	}
	
	/**
	 * Reads the inputstream given as first parameter to the length given as second parameter
	 * @param inputStream The stream to read from
	 * @param length The length of the stream
	 * @return An array of bytes that was retrieved from the stream
	 */
	private byte[] readFromStream(InputStream inputStream, int length)
	{
		readBytes = new byte[length];
		boolean done = false;
		int totalRead = 0;
		int counter = 0;
		try
		{		
			while(!done)
			{
				int justRead = inputStream.read(readBytes,totalRead,length-totalRead);
				totalRead = totalRead + justRead;
				if(totalRead == length)
					done = true;
				counter++;
			}
			/*The int returned by read() will always be -1. This call is made to avoid a 
			IOException on the sending node. Reading -1 will tell the sending node that
			the entire stream has been read.*/
			inputStream.read();
		} 
		catch (IOException e)
		{
			Log.getInstance().addElement(LogElementType.ERROR,e.getMessage());
		}
		try 
		{
			inputStream.close();
		} 
		catch (IOException e) 
		{
			Log.getInstance().addElement(LogElementType.ERROR,"Error closing stream: " + e.getMessage());
		}
		return readBytes;
	}
	
	/*
	 * Reads the inputstream given as first parameter to the length given as second parameter.
	 * This method can be used as an alternative to {@link #readFromStream(InputStream, int)}. 
	 * Testing have shown that the emulator reads the stream much slower with this method, but
	 * real phones do not show a significant difference.
	 * @param inputStream The stream to read from
	 * @param length The length of the stream
	 * @return An array of bytes that was retrieved from the stream
	 
	private byte[] readFromStreamBitByBit(InputStream inputStream, int length)
	{
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		try
		{
			int tmp = inputStream.read();
			while(tmp!=-1)
			{
				bos.write(tmp);
				tmp = inputStream.read();
			}
			//The int returned by read() will always be -1. This call is made to avoid a 
			//IOException on the sending node. Reading -1 will tell the sending node that
			//the entire stream has been read.
		} 
		catch (IOException e)
		{
			Log.getInstance().addElement(LogElementType.ERROR,e.getMessage());
		}
		try 
		{
			inputStream.close();
		} 
		catch (IOException e) 
		{
			Log.getInstance().addElement(LogElementType.ERROR,e.getMessage());
		}
		return bos.toByteArray();
	}*/
	
	/**
	 * Adds the nodes given as parameters to the default group if they are not allready in the group.
	 * When a node is added, the {@link peer2me.framework.FrameworkSubscriber#nodeDiscovered(Node)} is 
	 * inderictly invoked by this method. 
	 * @param linkedNodes The nodes that are added to the default group
	 */
	private void addNewNodesToDefaultGroup(SendableNodeInfo linkedNodes)
	{
		while(linkedNodes != null)
		{
			String nodeAddress = linkedNodes.getNodeAddress();
			//Making sure that the node does not add itself to the group
			if(!nodeAddress.equals(Session.getInstance().getLocalNode().getAddress()))
			{
				//If the node is not allready in the group, add it
				if(Session.getInstance().getNode(nodeAddress) == null)
				{
					InstanceOfRemoteDevice r = new InstanceOfRemoteDevice(linkedNodes.getNodeAddress());
					BluetoothNode node = new BluetoothNode(r,linkedNodes.getConnectionURL(),linkedNodes.getPingConnectionUrl(),linkedNodes.getNodeName());
					bluetooth.nodeFound(node);		
				}
			}
			linkedNodes = linkedNodes.getNextNode();
		}
	}
	
	/**
	 * Inherited method from javax.obex.ServerRequestHandler.
	 * Invoked when a remote device disconnects
	 * @param req The HeaderSet that is received upon disconnection
	 * @param resp The HeaderSet that can be sent upon disconnection 
	 */
	public void onDisconnect (HeaderSet req, HeaderSet resp) 
	{		
		Log.getInstance().addElement(LogElementType.INFORMATION, "Node disconnected");
	}

	/**
	 * If this class has received all the data that the remote device has sent, this class has fulfilled
	 * its purpose and is finished. 
	 * @return True if finished, false otherwise
	 */
	public boolean isFinished() 
	{
		return finished;
	}

	/**
	 * Sets a reference to the connection that is assosiated with this handler. This method is
	 * called by {@link BluetoothListener}.
	 * @param connection The connection that the data is being received on
	 */
	public void setConnection(Connection connection) 
	{
		this.connection = connection;
	}
}
