package peer2me.network.bluetooth;

import java.io.IOException;

import javax.bluetooth.LocalDevice;
import javax.bluetooth.RemoteDevice;

import peer2me.network.NetworkNode;

/**
 * This class is used to represent a remote Bluetooth device. This class extends NetworkNode and contains
 * information such as address, name, connectionURL, pingConnectionURL and a RemoteDevice representation
 * of the remote Bluetooth device
 * @author Kim Saxlund
 *
 */
public class BluetoothNode extends NetworkNode
{
	private String pingConnectionURL = "";
	
	/**
	 * Constructor. Used when the framework needs to create an instance of BluetoothNode of a remote 
	 * node after it has been found during a regular service search.
	 * @param remoteDevice The RemoteDevice that will be represented by this BluetoothNode
	 */
	public BluetoothNode(RemoteDevice remoteDevice)
	{
		super();
		try
		{
			this.name = remoteDevice.getFriendlyName(false);
		} 
		catch (IOException e)
		{
			this.name = "Unknown";
		}
		if(this.name==null)
			this.name = "Unknown";
		this.address = remoteDevice.getBluetoothAddress();
		this.object = remoteDevice;
	}
	
	/**
	 * Constructor. Used when the framework needs to create an instance of BluetoothNode of a 
	 * remote node A based on information received from a remote node B.
	 * @param remoteDevice The RemoteDevice that will be represented by this BluetoothNode
	 * @param connectionURL The connectionURL of the BluetoothNode
	 * @param pingConnectionURL The pingConnectionURL of the BluetoothNode
	 * @param name The name of the BluetoothNode
	 */
	public BluetoothNode(RemoteDevice remoteDevice, String connectionURL, String pingConnectionURL, String name)
	{
		super();
		this.name = name;

		if(this.name==null)
			this.name = "Unknown";
		this.address = remoteDevice.getBluetoothAddress();
		this.object = remoteDevice;
		this.connectionURL = connectionURL;
		this.pingConnectionURL = pingConnectionURL;
	}
	
	/**
	 * Constructor. Used when the framework needs to create an instance of BluetoothNode of the local 
	 * node for different purposes.
	 * @param localDevice The local node
	 */
	public BluetoothNode(LocalDevice localDevice)
	{
		super();
		this.name = localDevice.getFriendlyName();
		this.address = localDevice.getBluetoothAddress();
		this.object = localDevice;
	}	
	
	/**
	 * Returns the runtime class of object. That Class object is the object that is 
	 * locked by static synchronized methods of the represented class.
	 * @return The runtime class of object
	 */
	public Class getObjectClass()
	{
		return object.getClass();
	}
	
	/**
	 * Returns the value of the toString() method of the local variable object
	 * @return The value of the toString() method of the local variable object
	 */
	public String getObjectType()
	{
		return object.toString();
	}	
	
	/**
	 * Returns a neat string that can look something like this: "W800i at A154BC579E05"
	 * @return String A String describig the node
	 */
	public String toString()
	{
		return name+" at "+address;
	}	

	/**
	 * Sets the connectionURL of the BluetoothNode
	 * @param connectionURL The connectionURL
	 */
	public void setConnectionURL(String connectionURL)
	{
		this.connectionURL = connectionURL;
	}
	
	/**
	 * Gets the url that other nodes must use to ping the BluetoothNode
	 * @return The pingConnectionURL
	 */
	public String getPingConnectionURL()
	{
		return pingConnectionURL;
	}

	/**
	 * Sets the url that other nodes must use to ping the BluetoothNode
	 * @param pingConnectionURL The pingConnectionURL
	 */
	public void setPingConnectionURL(String pingConnectionURL)
	{
		this.pingConnectionURL = pingConnectionURL;
	}

}
