package peer2me.network.bluetooth;

import java.io.IOException;
import java.io.OutputStream;

import javax.microedition.io.Connection;
import javax.microedition.io.Connector;
import javax.obex.ClientSession;
import javax.obex.HeaderSet;
import javax.obex.Operation;

import peer2me.log.Log;
import peer2me.log.LogElementType;
import peer2me.message.Message;
import peer2me.message.MessagePart;
import peer2me.message.SendFileObject;
import peer2me.node.Node;

/**
 * This class does the actual connecting and sending of data to a remote device.
 * This class is used by {@link MessageQueueProcessor} and {@link peer2me.network.bluetooth.PingQueueProcessor}
 * @author Kim Saxlund
 *
 */
public class BluetoothObjectPush
{
	private Connection connection;
	private ClientSession clientSession;
	private HeaderSet headerSet;
	private Thread parentThread;
	private Bluetooth bluetooth;
	
	/**
	 * Constructor used by the PingQueueProcessor class
	 */
	protected BluetoothObjectPush()
	{
	}
	
	/**
	 * Constructor used by the MessageQueueProcessor class
	 * @param parentThread The MessageQueueProcessor thread
	 * @param bluetooth Reference to Bluetooth
	 */
	protected BluetoothObjectPush(Thread parentThread, Bluetooth bluetooth)
	{
		this.parentThread = parentThread;
		this.bluetooth = bluetooth;
	}

	/**
	 * Tries to create a connection to another Bluetooth device.
	 * @param connectionURL The connectionURL that is used to establish a connection
	 * @return True if a connection could be established, false otherwise
	 */
	public synchronized boolean connect(String connectionURL)
	{
		try
		{
			connection = Connector.open(connectionURL);
		} 
		catch (IOException e)
		{
			Log.getInstance().addElement(LogElementType.WARNING,"Could not connect: "+e.getMessage());
			return false;
		}
		clientSession = (ClientSession)connection;
		headerSet = clientSession.createHeaderSet();
		try
		{
			clientSession.connect(headerSet);
		} 
		catch (IOException e)
		{
			Log.getInstance().addElement(LogElementType.ERROR,"No client session: "+e.getMessage());
		}
		return true;
	}
	
	/**
	 * Closes the connection that was created by {@link #connect(String)}.
	 *
	 */
	public void closeConnection()
	{
		try
		{
			connection.close();
		} 
		catch (IOException e)
		{
			Log.getInstance().addElement(LogElementType.ERROR,"Unable to close connection");
			e.printStackTrace();
		}
	}
	
	/**
	 * Sends the message given as parameter. The recipiencts of the message lies in the message itself.
	 * First it will invoke {@link #connect(String)}, and if a connection is established, it
	 * will send the message.
	 * @param message The message to be sent
	 */
	public synchronized void sendMessage(Message message)
	{
		message.storeAllPrimitivesInOnePart();
		final Node[] recipients = message.getRecipients();
		
		for (int i = 0; i < recipients.length; i++) 
		{			
			String connectionURL = ((BluetoothNode)recipients[i].getNetworkNode()).getConnectionURL();
			if(!connect(connectionURL))
				continue;
			MessagePart[] parts = message.getAllMessageParts();
			headerSet.setHeader(Peer2MeHeaderSet.TOTAL_NUMBER_OF_MESSAGES, "" + message.numberOfChunksNeeded());
			headerSet.setHeader(Peer2MeHeaderSet.MESSAGE_ID, message.getMessageID());
			headerSet.setHeader(Peer2MeHeaderSet.SENDER_ADDRESS,message.getSender().getAddress());
			headerSet.setHeader(Peer2MeHeaderSet.SENDER_NAME,message.getSender().getNodename());
			
			int counter = 1;
			for (int j = 0; j < parts.length; j++) 
			{	
				MessagePart part = parts[j];
				if(part.getMessagePartType() == MessagePart.FILEOBJECT)
				{		
					/*Tricky part. When a file is attached to a message, it will be run as a Thread in order to
					 * prevent blocking. When the Midlet tries to read a local file it needs permission from the user.
					 * The phone automatically shows a popup on the screen and we can not send the message with the file
					 * until the user has allowed the application to read local files. That is why there is a "sleep" 
					 * below. Removing this will result in a Message being sent without a file and the framework will
					 * not function properly.
					 * */
					while(!((SendFileObject)part.getFileObject()).isAccessGranted())
					{
						try
						{
							Thread.sleep(100);
						} 
						catch (InterruptedException e)
						{}
					}
					
					headerSet.setHeader(Peer2MeHeaderSet.TOTAL_NUMBER_OF_MESSAGES, "" + message.numberOfChunksNeeded());
					SendFileObject sendFileObject = (SendFileObject)part.getFileObject();
					while(sendFileObject.getBytesLeft() > 0)
					{				
						byte[] bytes = sendFileObject.read();						
						
						if(sendFileObject.getBytesLeft() == 0)
							headerSet.setHeader(Peer2MeHeaderSet.END_OF_FILE,"Yes");
						else
							headerSet.setHeader(Peer2MeHeaderSet.END_OF_FILE,"No");
						
						headerSet.setHeader(Peer2MeHeaderSet.FILE_NAME,sendFileObject.getFileName());
						headerSet.setHeader(Peer2MeHeaderSet.MESSAGE_NUMBER, "" + (counter));						
						headerSet.setHeader(Peer2MeHeaderSet.LENGTH,"" + sendFileObject.getNumberOfBytesJustRead());
						headerSet.setHeader(Peer2MeHeaderSet.FILE_SIZE, "" + sendFileObject.getFileSize());
						headerSet.setHeader(Peer2MeHeaderSet.NAME,parts[j].getKey());
						headerSet.setHeader(Peer2MeHeaderSet.TYPE,Integer.toString(Message.MESSAGE_FILE));						
						headerSet.setHeader(Peer2MeHeaderSet.MESSAGE_PART_ID,part.getKey());						
						
						try
						{
							Operation operation = clientSession.put(headerSet);
							OutputStream outputStream = operation.openOutputStream();							
							outputStream.write(bytes);
							outputStream.flush();
							outputStream.close();
							operation.close();
							counter++;
						} 
						catch (IOException e)
						{
							Log.getInstance().addElement(LogElementType.ERROR,"Error sending:"+e.getMessage());
						}						
					}
				}
				else 
				{					
					byte[] bytes = parts[j].getObjectBytes();

					headerSet.setHeader(Peer2MeHeaderSet.MESSAGE_NUMBER, "" + (counter));					
					headerSet.setHeader(Peer2MeHeaderSet.NAME,parts[j].getKey());
					headerSet.setHeader(Peer2MeHeaderSet.LENGTH,"" + bytes.length);
					headerSet.setHeader(Peer2MeHeaderSet.TYPE,Integer.toString(message.getMessageType()));
					try
					{
						Operation operation = clientSession.put(headerSet);
						OutputStream outputStream = operation.openOutputStream();
						outputStream.write(bytes);
						outputStream.flush();
						outputStream.close();
						operation.close();
						counter++;
					} 
					catch (IOException e)
					{
						Log.getInstance().addElement(LogElementType.ERROR,"Error sending:"+e.getMessage());
					}
				}
			}
			try 
			{
				connection.close();
				synchronized (parentThread) 
				{
					parentThread.notify();
				}
			}
			catch (IOException e) 
			{
				Log.getInstance().addElement(LogElementType.INFORMATION,"Error closing: " + e.getMessage());
			}
		}
		if(message.getMessageType() == Message.MESSAGE_NODE_LEFT)
		{
			bluetooth.setGoodbyeSent(true);
		}
	}
}
