package peer2me.network.bluetooth;

import java.io.IOException;

import javax.bluetooth.BluetoothStateException;
import javax.bluetooth.DiscoveryAgent;
import javax.bluetooth.LocalDevice;
import javax.bluetooth.ServiceRecord;
import javax.bluetooth.UUID;
import javax.microedition.io.Connection;
import javax.microedition.io.Connector;
import javax.obex.SessionNotifier;

import peer2me.log.Log;
import peer2me.log.LogElementType;
import peer2me.service.Service;

/**
 * This class creates and opens up a connection so that other devices can establish a connection.
 * The connections are only used to see if the local node is alive. No data is ever transferred
 * on this connection
 * @author Kim Saxlund
 *
 */
public class BluetoothPingListener implements Runnable
{
	private Bluetooth bluetooth;
	private Service service;
	
	private boolean run;
	private Thread thread;
	
	/**
	 * Constructor that sets some local variables and starts the thread
	 * @param bluetooth A reference to Bluetooth, which created an instance of this class
	 * @param service The service contains necessary information to open up a connection 
	 */
	public BluetoothPingListener(Bluetooth bluetooth, Service service) 
	{
		this.bluetooth = bluetooth;
		this.service = service;
		run = true;
		
		thread = new Thread(this);
		thread.start();
	}
	
	/**
	 * Opens up a connection. If someone connects, it creates an instance
	 * of {@link BluetoothConnectionHandler} which is assositated with the connection. It will not
	 * open up a new connection until the current connection is closed.
	 */
	public void run()
	{
		UUID uuid = new UUID(service.getHexServiceID(),false);
		String url = "btgoep://localhost:" + uuid + ";name="+service.getAsciiPingServiceID()+";authenticate=false;encrypt=false";
		SessionNotifier sessionNotifier = null;
		
		try
		{
			LocalDevice localDevice = LocalDevice.getLocalDevice();
			localDevice.setDiscoverable(DiscoveryAgent.GIAC);
			sessionNotifier = (SessionNotifier)Connector.open(url);
			String localConnectionURL = localDevice.getRecord(sessionNotifier).getConnectionURL(ServiceRecord.NOAUTHENTICATE_NOENCRYPT,false);
			bluetooth.setLocalPingConnectionURL(localConnectionURL);
		} 
		catch (BluetoothStateException e)
		{
			Log.getInstance().addElement(LogElementType.ERROR,"BluetoothPingListener_run():"+e.getMessage());
		} 
		catch (IOException e)
		{
			Log.getInstance().addElement(LogElementType.ERROR,"BluetoothPingListener_run():"+e.getMessage());
		}
		Connection connection = null;
		while(run)
		{
			try
			{
				BluetoothPingConnectionHandler handler = new BluetoothPingConnectionHandler(connection,thread);
				Log.getInstance().addElement(LogElementType.INFORMATION,"Listening for ping");
				connection = sessionNotifier.acceptAndOpen(handler);
				handler.setConnection(connection);
				synchronized (thread) 
				{
					try 
					{
						thread.wait();
					} 
					catch (InterruptedException e) 
					{
						Log.getInstance().addElement(LogElementType.ERROR,e.getMessage());
					}
				}
			} 
			catch (IOException e)
			{
				Log.getInstance().addElement(LogElementType.ERROR,"BluetoothListener_run():"+e.getMessage());
			}
		}		
	}
}
