package peer2me.network.bluetooth;

import java.util.Vector;

import javax.bluetooth.BluetoothStateException;
import javax.bluetooth.DataElement;
import javax.bluetooth.DeviceClass;
import javax.bluetooth.DiscoveryAgent;
import javax.bluetooth.DiscoveryListener;
import javax.bluetooth.LocalDevice;
import javax.bluetooth.RemoteDevice;
import javax.bluetooth.ServiceRecord;
import javax.bluetooth.UUID;

import peer2me.message.Message;
import peer2me.message.SendableNodeInfo;
import peer2me.node.Node;
import peer2me.service.Service;
import peer2me.session.Session;
import peer2me.log.Log;
import peer2me.log.LogElementType;

/**
 * The purpose of this class is to search for nearby Bluetooth devices that runs the 
 * same application as the local node.
 * @author Kim Saxlund
 *
 */
public class BluetoothSearcher extends Thread implements DiscoveryListener
{

	private Bluetooth bluetooth;
	private LocalDevice localDevice;
	private DiscoveryAgent discoveryAgent;
	
	private Vector discoveredDevices;
	
	private UUID[]  uuids = new UUID[1];
	
	private int[] attributes = { 0x100,0x101,0x102};
	private int mobileDeviceClassCode = 0x200;
	private String serviceID;
	private String pingServiceID;
	private int discoveryMode;
	
	/**
	 * Constructor. Starts this thread
	 * @param bluetooth The class that created an instance of this class
	 * @param service The service that will be searched for
	 */
	protected BluetoothSearcher(Bluetooth bluetooth, Service service) 
	{
		this.bluetooth = bluetooth;
		discoveredDevices = new Vector();
		serviceID = service.getASCIIServiceID();
		pingServiceID = service.getAsciiPingServiceID();
		uuids[0] = new UUID(service.getHexServiceID(),false);
		
		this.start();
	}
	
	/**
	 * Initializes and sets the discoveryMode and retrieves the DiscoveryAgent from the LocalDevice.
	 * Must be called before a {@link #search()} is performed.
	 *
	 */
	protected void initialize()
	{
		int tries = 0;
		while(tries < 3)
		{
			try
			{
				localDevice = LocalDevice.getLocalDevice();
				discoveryAgent = localDevice.getDiscoveryAgent();
				break;
			} 
			catch (BluetoothStateException e)
			{
				Log.getInstance().addElement(LogElementType.WARNING,"Couldn't get localdevice");
				tries++;
				continue;	
			}
		}
		discoveryMode = DiscoveryAgent.GIAC;
	}
	
	/**
	 * Starts a search for other Bluetooth devices.
	 *
	 */
	public void search()
	{
		try
		{
			discoveryAgent.startInquiry(discoveryMode,this);			
		} 
		catch (BluetoothStateException e)
		{
			Log.getInstance().addElement(LogElementType.ERROR,e.getMessage());
		}
	}
	
	/**
	 * Required method from DiscoveryListener. Is invoked when a device is discovered
	 * @param deviceClass The type of Bluetooth device that was discovered
	 * @param remoteDevice A representation of the device
	 */
	public void deviceDiscovered(RemoteDevice remoteDevice, DeviceClass deviceClass)
	{
		int majorDeviceClass = deviceClass.getMajorDeviceClass();
		if(majorDeviceClass == mobileDeviceClassCode)
		{
			if(!discoveredDevices.contains(remoteDevice))
				discoveredDevices.addElement(remoteDevice);
		}		
	}

	/**
	 * Required method from DiscoveryListener. Is invoked when a device discovery has completed.
	 * A search for services on the discovered devices is initiated.
	 */
	public void inquiryCompleted(int returnValue)
	{
		String inqStatus = "";
		switch (returnValue)
		{
		case DiscoveryListener.INQUIRY_COMPLETED:
			if(discoveredDevices.size()==0)
			{	
				inqStatus = "No devices found";
				bluetooth.searchCompleted();
        	}    	
        	else
        	{	
        		try 
        		{
    				discoveryAgent.searchServices(attributes,uuids,(RemoteDevice)discoveredDevices.firstElement(),this);
    				discoveredDevices.removeElementAt(0);
    			} 
        		catch (BluetoothStateException e) 
        		{
        			Log.getInstance().addElement(LogElementType.INFORMATION,"B.StateExc: " + e.getMessage());
    			}
    		}
			break;
		case DiscoveryListener.INQUIRY_TERMINATED:
			inqStatus = "Inquiry terminated";
			break;
		case DiscoveryListener.INQUIRY_ERROR:
			inqStatus = "Inquiry error";
			break;
		default:
			break;
		}
		inqStatus.toString();
	}
	
	/**
	 * Required method from DiscoveryListener.
	 * Is invoked when a service is found on a remote Bluetooth device. 
	 * If the device has the service we are looking for, a {@link BluetoothNode} will be created
	 * to represent the device and the node is added to the local group. The 
	 * {@link peer2me.framework.FrameworkSubscriber#nodeDiscovered(Node)}
	 * is also invoked, informing the application about the discovered device.
	 * @param serviceRecords The services that were found on the device
	 * @param transID The transaction ID of the service search that is posting the result
	 */
	public void servicesDiscovered(int transID, ServiceRecord[] serviceRecords)
	{
		RemoteDevice remoteDevice = null;
		BluetoothNode bluetoothNode = null;
		if(serviceRecords.length > 0)
		{
			remoteDevice = serviceRecords[0].getHostDevice();
			bluetoothNode = new BluetoothNode(remoteDevice);
			for (int i = 0; i < serviceRecords.length; i++)
			{
				DataElement serviceNameElement = serviceRecords[i].getAttributeValue(attributes[0]);
				Log.getInstance().addElement(LogElementType.INFORMATION,"A service: " + serviceNameElement.getValue());
				
				if(serviceNameElement.getValue().equals(serviceID))
				{
					Log.getInstance().addElement(LogElementType.INFORMATION,"Found matching service");
					String url = serviceRecords[i].getConnectionURL(ServiceRecord.NOAUTHENTICATE_NOENCRYPT,false);
					
					Node node = Session.getInstance().getNode(bluetoothNode.getAddress());
					if(node == null)
					{
						bluetoothNode.setConnectionURL(url);
						bluetooth.nodeFound(bluetoothNode);
					}
					else
					{
						BluetoothNode blueNode = (BluetoothNode)node.getNetworkNode();
						blueNode.setConnectionURL(url);
					}
				}			
				else if(serviceNameElement.getValue().equals(pingServiceID))
				{
					Log.getInstance().addElement(LogElementType.INFORMATION,"Found pingservice");
					String url = serviceRecords[i].getConnectionURL(ServiceRecord.NOAUTHENTICATE_NOENCRYPT,false);
					Node node = Session.getInstance().getNode(bluetoothNode.getAddress());
					if(node == null)
					{
						bluetoothNode.setPingConnectionURL(url);
						bluetooth.nodeFound(bluetoothNode);
					}
					else
					{
						BluetoothNode blueNode = (BluetoothNode)node.getNetworkNode();
						blueNode.setPingConnectionURL(url);
					}
				}
			}
		}	
	}

	/**
	 * Required method from DiscoveryListener. Called when a service search is 
	 * completed or was terminated because of an error.
	 * Continues to perform a service search on the remaining devices. If there are no more devices
	 * the search is entirely completed and {@link peer2me.framework.FrameworkSubscriber#searchCompleted()}
	 * is invoked. The ping is also activated
	 * @param transID The transaction ID identifying the request which initiated the service search
	 * @param respCode The response code that indicates the status of the transaction
	 */
	public void serviceSearchCompleted(int transID, int respCode)
	{
		if(discoveredDevices.size()>0)
		{
    		try 
    		{
    			discoveryAgent.searchServices(attributes,uuids,(RemoteDevice)discoveredDevices.firstElement(),this);
    			discoveredDevices.removeElementAt(0);
    		} 
    		catch (BluetoothStateException e) 
    		{
    			Log.getInstance().addElement(LogElementType.ERROR,"B.StateExc: " + e.getMessage());
    		}
    	}
    	else
    	{    	
    		bluetooth.searchCompleted();
    		Log.getInstance().addElement(LogElementType.INFORMATION,"Distributing nodeInfo");
    		distributeNodeInformation();
    		bluetooth.activatePing();
    	}
    	
		String status = "";
    	switch(respCode) 
    	{
        	case DiscoveryListener.SERVICE_SEARCH_COMPLETED:     
                	status = "service discovery completed \n";               		               		
                    break;
                    
                case DiscoveryListener.SERVICE_SEARCH_DEVICE_NOT_REACHABLE:
                	status = "search device not reachable\n";
                    break;
                    
                case DiscoveryListener.SERVICE_SEARCH_ERROR:
                	status = "service search error\n";
                    break;
                    
                case DiscoveryListener.SERVICE_SEARCH_NO_RECORDS:
                	status = "service search: no records found\n";
                    break;
                    
                case DiscoveryListener.SERVICE_SEARCH_TERMINATED:
                	status = "service search terminated\n";
                    break;
    	}
    	status.toString();
	}

	/**
	 * Distributes information about all the found nodes. This way, all the other nodes are
	 * informed about each other and they do not have to perform a search to connect to each other.
	 *
	 */
	public void distributeNodeInformation()
	{
		SendableNodeInfo linkedNodeList = null;
		SendableNodeInfo head = null;
		Node allNodes[] = Session.getInstance().getAllNodes();
		for (int i = 0; i < allNodes.length; i++)
		{
			SendableNodeInfo node = null;
			if(allNodes[i].getAddress().equalsIgnoreCase(Session.getInstance().getLocalDevice().getBluetoothAddress()))
			{
				//Adding local node to linked node list
				Node thisNode = Session.getInstance().getLocalNode();
				node = new SendableNodeInfo(thisNode.getNodename(),thisNode.getAddress(),bluetooth.getLocalConnectionURL(),bluetooth.getLocalPingConnectionURL());
			}
			else
			{
				BluetoothNode bluetoothNode = (BluetoothNode)allNodes[i].getNetworkNode();
				node = new SendableNodeInfo(allNodes[i].getNodename(),
						allNodes[i].getAddress(),bluetoothNode.getConnectionURL(),bluetoothNode.getPingConnectionURL());
			}	
			if(head == null)
			{	
				head = node;
				linkedNodeList = head;
			}
			else
			{
				linkedNodeList.setNextNode(node);
				linkedNodeList = node;
			}
		}
		
		Message message = new Message();
		message.setMessageType(Message.MESSAGE_NODE_JOIN);
		message.addSerializedObject("nodeList", head);
		message.addRecipients(allNodes);
		bluetooth.sendMessage(message);
	}
}
