package peer2me.network.bluetooth;

import peer2me.network.bluetooth.Bluetooth;
import peer2me.network.bluetooth.BluetoothObjectPush;
import peer2me.log.Log;
import peer2me.log.LogElementType;

/**
 * This class processes the messagequeue that is located in {@link peer2me.network.bluetooth.Bluetooth}.
 * This class constantly watches the queue, and when a message is added to the queue, this class 
 * retrieves the message and sends the message through {@link peer2me.network.bluetooth.BluetoothObjectPush}
 * @author Kim Saxlund
 *
 */
public class MessageQueueProcessor implements Runnable
{
	Bluetooth bluetooth;
	boolean processQueue = true;
	private Thread thisThread;
	
	/**
	 * Constructor. Starts this thread.
	 * @param bluetooth Reference to the Bluetooth class
	 */
	protected MessageQueueProcessor(Bluetooth bluetooth)
	{
		this.bluetooth = bluetooth;
		thisThread = new Thread(this);
		thisThread.start();
	}
	
	/**
	 * Runs as long as the private boolean <code>processQueue</code> is true. 
	 * Checks the message queue at regular intervals. When a message appears, this
	 * method creates an instance of BluetoothObjectPush which sends the message
	 */
	public synchronized void run()
	{
		while(processQueue)
		{
			while(bluetooth.getMessageQue().isEmpty())
			{
				try
				{
					Thread.sleep(100);
				} catch (InterruptedException e)
				{}
			}
			Log.getInstance().addElement(LogElementType.INFORMATION, "Processing message");
			final Bluetooth tmpBluetooth = bluetooth;
			final Thread innerThread = new Thread(new Runnable() 
			{
				public void run() 
				{
					BluetoothObjectPush bp = new BluetoothObjectPush(thisThread,tmpBluetooth);
					bp.sendMessage(tmpBluetooth.getMessageQue().getFirstMessage());
				}
			});
			innerThread.start();
		    try 
		    {
				synchronized (thisThread) 
				{
					thisThread.wait();
				}
			} catch (InterruptedException e) 
			{
				Log.getInstance().addElement(LogElementType.ERROR,e.getMessage());
			}
		    Log.getInstance().addElement(LogElementType.INFORMATION, "Message processed");
		}
	}
	
	/**
	 * If the queue is being checked at regular intervals, this method returns true
	 * @return True if queue is being checked, false otherwise
	 */
	public boolean isProcessQueue()
	{
		return processQueue;
	}

	/**
	 * Can be used to stop this thread. Use 'false' as parameter, and this thread will stop
	 * checking the queue, and finish
	 * @param processQue Use false to stop checking the queue
	 */
	public void setProcessQueue(boolean processQue)
	{
		this.processQueue = processQue;
	}
}
