package peer2me.node;

import peer2me.network.NetworkNode;

/**
 * A simple class for storing information
 * about a node, based on a {@link peer2me.network.NetworkNode}.
 * Contains also statistics regarding ping/eche detection
 * @author Tommy Bjoernsgaard
 *
 */
public class Node 
{
	private String address;
	private String nodename;
	private NetworkNode networkNode;
	private int ping;
	private int echo;
	private int lastdiff;
	private static final int MAX_DIFF = 5;
	
	/**
	 * Intanciates a new Node based on a {@link peer2me.network.NetworkNode}
	 * @param networkNode the object node is based on
	 */
	public Node(NetworkNode networkNode)
	{		
		this.nodename = networkNode.getName();
		this.address = networkNode.getAddress();
		this.networkNode = networkNode;
		ping = 0;
		echo = 0;
		lastdiff = 0;
	}
	
	/**
	 * Gets the address of the node
	 * @return the address of the node
	 */
	public String getAddress()
	{
		return address;		
	}
	
	/**
	 * Gets the name of the node
	 * @return the name of the node
	 */
	public String getNodename()
	{
		return nodename;
	}

	/**
	 * Gets the {@link peer2me.network.NetworkNode} the node
	 * is based on.
	 * @return the NetworkNode
	 */
	public NetworkNode getNetworkNode()
	{
		return networkNode;
	}

	/**
	 * Returns the name of the node and its address
	 * @return Nodename and address
	 */
	public String toString()
	{
		return nodename+" at "+address;
	}

	/**
	 * Gets the number of pings the node has
	 * responded to; number of echoes.
	 * @return the number of echoes
	 */
	public int getEcho() 
	{
		return echo;
	}

	/**
	 * Increments the echo-counter by 1
	 *
	 */
	public void incrementEcho() 
	{
		this.echo++;
	}

	/**
	 * Gets the number of ping-messages sent by the node
	 * @return the number of pings.
	 */
	public int getPing() 
	{
		return ping;
	}

	/**
	 * Increments the ping-counter by 1
	 *
	 */
	public void incrementPing() 
	{
		this.ping++;
	}
	
	/**
	 * Compares the number of pings sent and echoes received,
	 * comparing to see if the difference is larger than
	 * <code>MAX_DIFF</code>.
	 * @return true if the diff is to large and node is abscent, else false.
	 */
	public boolean isNodeAbscent()
	{
		int diff = ping-echo;
		if(lastdiff == diff)
		{
			lastdiff = 0;
			echo = ping;
		}
		else
		{
			lastdiff = diff;
		}

		if(diff >= MAX_DIFF)
			return true;
		else
			return false;
	}
}
