package peer2me.session;

import java.util.Vector;

import javax.bluetooth.BluetoothStateException;
import javax.bluetooth.LocalDevice;

import peer2me.exception.FrameworkNotInitializedException;
import peer2me.exception.GroupNotFoundException;
import peer2me.exception.LocalDeviceNotFoundException;
import peer2me.exception.NodeNotFoundException;
import peer2me.framework.Framework;
import peer2me.group.Group;
import peer2me.log.Log;
import peer2me.log.LogElementType;
import peer2me.message.Message;
import peer2me.network.Network;
import peer2me.network.NetworkNode;
import peer2me.network.bluetooth.BluetoothNode;
import peer2me.node.Node;
import peer2me.service.Service;

/**
 * This class binds togethers most of the framework and acts as a
 * global access point. Initiates the {@link peer2me.network.Network}, creates a
 * {@link peer2me.service.Service}, and sets the default {@link peer2me.group.Group}.
 * Uses Singelton. <code>getInstance(...)</code> will be used only one time.
 * After that <code>getInstance()</code> should be used.
 * @author Tommy Bjoernsgaard
 *
 */
public class Session 
{
	private static Session singleton;
	
	private Framework framework;
	private Group group;
	private Service service;
	private Network network;
	private LocalDevice localDevice;
	private Node localNode;
	private Vector groups;
	
	private boolean initialized;
	
	private Session()
	{	
		groups = new Vector();
		initialized = false;
	}
	
	/**
	 * This should be used only one time, and sets a default {@link peer2me.group.Group},
	 * the {@link peer2me.network.Network} and the {@link peer2me.service.Service}. Will also set
	 * local properties as <code>localDevice</code> and <code>localNode</code>.
	 * @param framework a reference to framework.
	 * @param group a group which is set as default
	 * @param service the service provided by framework
	 * @param network the network provided by framework
	 * @param activatePing sets if ping-functionality should be used
	 * @return Returns an instance of Session
	 */
	public static synchronized Session getInstance(Framework framework, Group group, Service service, Network network, boolean activatePing)
	{
		if(singleton == null)
		{
			Log.getInstance().addElement(LogElementType.INFORMATION,"Created Session");
			singleton = new Session();
			singleton.setFramework(framework);
			singleton.setDefaultGroup(group);
			singleton.setService(service);
			singleton.setNetwork(network);
			singleton.setActivatePing(activatePing);
			singleton.setLocalDevice();
			singleton.setLocalNode();
		}
		return singleton;
	}
	
	/**
	 * Must only be used after <code>getInstance(...)</code> has be called one time.
	 * Acts as a global access point to Session
	 * @return a reference to Session
	 */
	public static synchronized Session getInstance()
	{
		if(singleton == null)
		{
			throw new NullPointerException("Session not set. Use getInstance(Group,Service,Network) first!");
		}
		return singleton;
	}
	
	/**
	 * Initializes the network.
	 *
	 */
	public void initialize()
	{
		network.initialize();
		initialized = true;
	}
	
	/**
	 * Performes a search for devices and services.
	 * <code>initialize</code> must be called before trying to search.
	 * @throws FrameworkNotInitializedException
	 */
	public void search() throws FrameworkNotInitializedException
	{
		Log.getInstance().addElement(LogElementType.INFORMATION,"Searching for devices");
		if(initialized)
		{
			if(!network.search())
				Log.getInstance().addElement(LogElementType.WARNING, "Sorry, no searching right now. Busy");
		}else
			throw new FrameworkNotInitializedException();
	}

	/**
	 * This method is called whenever a new device running Peer2Me is found.
	 * {@link peer2me.network.NetworkNode} is a representative for this device.
	 * It creates a new {@link peer2me.node.Node} and sends it up to {@link peer2me.framework.Framework}.
	 * The new node will also be added to the default group. 
	 * @param networkNode the networkNode that is found
	 */
	public void nodeFound(NetworkNode networkNode)
	{
		Node node = new Node(networkNode);
		framework.nodeFound(node);
		Log.getInstance().addElement(LogElementType.INFORMATION,"Adding "+node.getNodename()+" to group "+group.getName());
		group.addNode(node);
	}
	
	/**
	 * This mehtod is called whenever a {@link peer2me.node.Node} is lost or abscent. It is then
	 * removed from its {@link peer2me.group.Group}. 
	 * @param node the node which is lost or abscent
	 */
	public void nodeLost(Node node)
	{
		Group thisgroup = getGroup(node);
		thisgroup.removeNode(node);
		Log.getInstance().addElement(LogElementType.INFORMATION,"Lost node "+node.toString()+" from group "+thisgroup.getName());
		framework.nodeLost(node);
	}
	
	/**
	 * This method is called when a search has been completed
	 *
	 */
	public void searchCompleted()
	{
		framework.searchCompleted();
	}
		
	/**
	 * This methods sets the default {@link peer2me.group.Group}. Also adds the group
	 * to the list of groups.
	 * @param group the group to be set default
	 */
	private void setDefaultGroup(Group group) 
	{
		this.group = group;
		groups.addElement(group);
	}
	
	/**
	 * Gets a referecnce to {@link peer2me.network.Network}
	 * @return a reference to network
	 */
	public Network getNetwork() 
	{
		return network;
	}
	
	/**
	 * Sets the network.
	 * @param network the network which should be used. Will replace the old one.
	 */
	private void setNetwork(Network network) 
	{
		this.network = network;
	}
	
	/**
	 * Sets if the ping-functionality should be activated or not
	 * @param activatePing true if the ping should be used, else false
	 */
	private void setActivatePing(boolean activatePing)
	{
		network.setActivatePing(activatePing);
	}
	
	/**
	 * Gets a reference to {@link peer2me.service.Service}
	 * @return the reference to service
	 */
	public Service getService() 
	{
		return service;
	}
	
	/**
	 * Sets the {@link peer2me.service.Service} which should be used. 
	 * Will replace the service already sat.
	 * @param service the new service
	 */
	private void setService(Service service) 
	{
		this.service = service;
	}

	/**
	 * Gets the {@link javax.bluetooth.LocalDevice}
	 * @return the localdevice object
	 */
	public LocalDevice getLocalDevice()
	{
		return localDevice;
	}
	
	/**
	 * Sets the {@link javax.bluetooth.LocalDevice}. This is used
	 * frequently to identify the local device running Peer2Me.
	 * Retrieves important info from the Bluetooth radio.
	 * @throws LocalDeviceNotFoundException
	 */
	private void setLocalDevice() throws LocalDeviceNotFoundException
	{
		try 
		{
			localDevice = LocalDevice.getLocalDevice();		
		} 
		catch (BluetoothStateException e) 
		{
			Log.getInstance().addElement(LogElementType.ERROR,"Could not set localDevice in Session. LocalDevice set to null");
			localDevice = null;
			throw new LocalDeviceNotFoundException();
		}		
	}

	/**
	 * Sets the local node.
	 *
	 */
	private void setLocalNode()
	{
		localNode = new Node(new BluetoothNode(localDevice));
		group.addNode(localNode);
	}
	
	/**
	 * Gets the local {@link peer2me.node.Node}
	 * @return the node
	 */
	public Node getLocalNode() 
	{
		return localNode;
	}
	
	/**
	 * Sets a local reference to the {@link peer2me.framework.Framework} class.
	 * @param framework the reference to framework
	 */
	private void setFramework(Framework framework)
	{
		this.framework = framework;
	}
	
	/**
	 * This method is called whenever a {@link peer2me.message.Message} is received.
	 * @param message the message which is received
	 */
	public void messageReceived(Message message)
	{
		framework.messageReceived(message);
	}
	
	/**
	 * This method is called for every messagepart that is received, giving
	 * the messagepartnumber of the total number of messages.
	 * @param part
	 * @param total
	 */
	public void messagePartReceived(String messageID, int part, int total)
	{
		framework.messagePartReceived(messageID, part, total);
	}

	/**
	 * This method creates a new {@link peer2me.group.Group} and adds it to 
	 * the session.
	 * @param name name of the group
	 * @return the group just created
	 */
	public Group createGroup(String name)
	{
		Group group = new Group(name);
		groups.addElement(group);
		return group;
	}
	
	/**
	 * This method creates a {@link peer2me.group.Group} and adds it to
	 * the session. Adds also an array of nodes directly.
	 * @param name the name of the group
	 * @param nodes the nodes added upon creation of the group
	 * @return the group just created
	 */
	public Group createGroup(String name, Node[] nodes)
	{
		Group  group = new Group(name, nodes);
		groups.addElement(group);
		return group;
	}
	
	/**
	 * Gets all the groups
	 * @return an array of the groups
	 */
	public Group[] getGroups()
	{
		Group[] g = new Group[groups.size()];
		groups.copyInto(g);
		return g;
	}
	
	/**
	 * Gets a {@link peer2me.group.Group} with the corresponding name. Throws an exception if
	 * the group is not found.
	 * @param name the name of the group
	 * @return the group
	 * @throws GroupNotFoundException
	 */
	public Group getGroup(String name) throws GroupNotFoundException
	{
		for (int i = 0; i < groups.size(); i++)
		{
			if(((Group)groups.elementAt(i)).getName().equals(name))
				return (Group)groups.elementAt(i);
		}
		throw new GroupNotFoundException();
	}

	/**
	 * Gets a {@link peer2me.group.Group} based on a {@link peer2me.node.Node}.
	 * It will get the group the node is a member of.
	 * @param node the node in the group
	 * @return the group with the node
	 * @throws GroupNotFoundException
	 */
	public Group getGroup(Node node)throws GroupNotFoundException	
	{
		Group[] allGroups = getGroups();
		for (int i = 0; i < allGroups.length; i++)
		{
			Node[] nodes = getAllNodes((allGroups[i]));
			for (int j = 0; j < nodes.length; j++)
			{
				if(nodes[j] == node)
					return allGroups[i];
			}
		}
		throw new GroupNotFoundException();
	}
	
	/**
	 * Gets the {@link peer2me.group.Group} at the given index in the collection of
	 * groups in session. Throws an exception if the group is not found.
	 * @param index the index of the group
	 * @return the group found at the index
	 * @throws GroupNotFoundException
	 */
	public Group getGroupAt(int index) throws GroupNotFoundException
	{
		if(index < groups.size())
			return (Group)groups.elementAt(index);
		else
			throw new GroupNotFoundException();
	}
	
	/**
	 * Removes the {@link peer2me.group.Group} at the given index. Will also
	 * remove the nodes in the group if not empty. Throws an exception if the group
	 * is not found.
	 * @param index the index of the group
	 * @throws GroupNotFoundException
	 */
	public void removeGroup(int index) throws GroupNotFoundException
	{
		if(index < groups.size())
			groups.removeElementAt(index);
		else
			throw new GroupNotFoundException();
	}
	
	/**
	 * Removes the {@link peer2me.group.Group} with the given name. Will also
	 * remove the nodes in the group if not empty. Throws an exception if the group
	 * is not found.
	 * @param name the name of the group
	 * @throws GroupNotFoundException
	 */
	public void removeGroup(String name) throws GroupNotFoundException
	{
			Group g = this.getGroup(name);
			groups.removeElement(g);
	}
	
	/**
	 * Gets all the discovered nodes 
	 * @return an array of the nodes
	 */
	public Node[] getAllNodes()
	{
		Vector v = new Vector();
		Group[] g = getGroups();
		Node[] n;
		for (int i = 0; i < g.length; i++)
		{
			n = g[i].getNodes();
			
			for (int j = 0; j < n.length; j++)
			{
					v.addElement(n[j]);			
			}
		}		
		n = new Node[v.size()];
		v.copyInto(n);
		v = null;
		return n;
	}

	/**
	 * Gets alle the nodes in the given group.
	 * @param group the group of the nodes
	 * @return an array of nodes
	 */
	public Node[] getAllNodes(Group group)
	{
		return group.getNodes();
	}
	
	/**
	 * Gets a {@link peer2me.node.Node} based on an address
	 * @param address the address of the node
	 * @return the node with the given address
	 */
	public Node getNode(String address) //throws NodeNotFoundException
	{
		Node[] n = getAllNodes();
		for (int i = 0; i < n.length; i++)
		{
			if(n[i].getAddress().equals(address))
				return n[i];
		}
		return null;
	}
	
	/**
	 * Gets the {@link peer2me.node.Node} at the given <code>index</code>
	 * in the given {@link peer2me.group.Group}. Throws an exception if
	 * the node is not found.
	 * @param index the index of the node
	 * @param group the group of the node
	 * @return the node
	 * @throws NodeNotFoundException
	 */
	public Node getNode(int index, Group group) throws NodeNotFoundException
	{
		return group.getNodeAt(index);
	}
	
	/**
	 * Moves a {@link peer2me.node.Node} from one {@link peer2me.group.Group}
	 * to another. Will also send a notification to all other nodes about the
	 * movement.  
	 * @param node the node to move
	 * @param from the group the node lies in
	 * @param to the new group of the node
	 */
	public void moveNode(Node node, Group from, Group to)
	{
		from.removeNode(node);
		to.addNode(node);
		
		//Gives the other connected nodes notification of the moved node
		Message message = new Message();
		message.addElement("node",node.getAddress());
		message.addElement("from",from.getName());
		message.addElement("to",to.getName());
		message.setMessageType(Message.MESSAGE_NODE_MOVE);
		message.addRecipients(getAllNodes());
		
		sendMessage(message);
	}
	
	/**
	 * Removes a {@link peer2me.node.Node} with the given <code>index</code> from a {@link peer2me.group.Group}. 
	 * Throws an exception if the node is not found.
	 * @param index the index of the node
	 * @param group the group of node
	 * @throws NodeNotFoundException
	 */
	public void removeNode(int index, Group group) throws NodeNotFoundException
	{
			group.removeNodeAt(index);
	}
	
	/**
	 * Removes a {@link peer2me.node.Node} from a {@link peer2me.group.Group}.
	 * @param node the node to remove
	 * @param group the group of the node
	 */
	public void removeNode(Node node, Group group)
	{
		group.removeNode(node);
	}
	
	/**
	 * Passes on a {@link peer2me.message.Message} to {@link peer2me.network.Network}. Used when sending messages.
	 * @param message the message to be sent.
	 */
	public void sendMessage(Message message)
	{
		network.sendMessage(message);
	}
}
